'''\
An asynchronous shell for experimenting with pulsar on the command line.
The shell is available for posix operative systems only. Windows support
to come at some point.

To use write a little script, lets call it ``pshell.py``::

    from pulsar.apps.shell import PulsarShell 
    
    if __name__ == '__main__':
        PulsarShell().start()

And run it::

    python pshell.py
    
The shell has already ``pulsar``, ``arbiter`` and the :class:`Actor`
class in the global dictionary::

    >> pulsar.__version__
    0.2.0
    >> arbiter.is_arbiter()
    True
    >> arbiter.state
    'running'
    >> arbiter.MANAGED_ACTORS
    {}
    >> a = arbiter.spawn(Actor)
    >> a.is_alive()
    True
    >> arbiter.MANAGED_ACTORS
    {'3a67a186': 3a67a186}
    >> arbiter.close_actors()
    >> a.is_alive()
    False
    
'''
import os
import sys
import code
from time import time

import pulsar
from pulsar.apps import tasks


if pulsar.ispy3k:
    def decode_line(line):
        return line
else:   #pragma    nocover
    def decode_line(line):
        encoding = getattr(sys.stdin, "encoding", None)
        if encoding and not isinstance(line, unicode):
            line = line.decode(encoding)
        return line
    

class InteractiveConsole(code.InteractiveConsole):  #pragma    nocover

    def pulsar_banner(self):
        cprt = 'Type "help", "copyright", "credits" or "license" for more information.'
        return "Python %s on %s\n%s\nPulsar %s\n" %\
                    (sys.version, sys.platform, cprt,pulsar.__version__)
     
    def setup(self, banner=None):
        """Closely emulate the interactive Python console.

        The optional banner argument specify the banner to print
        before the first interaction; by default it prints a banner
        similar to the one printed by the real Python interpreter,
        followed by the current class name in parentheses (so as not
        to confuse this with the real interpreter -- since it's so
        close!).

        """
        try:
            sys.ps1
        except AttributeError:
            sys.ps1 = ">>> "
        try:
            sys.ps2
        except AttributeError:
            sys.ps2 = "... "
        if banner is None:
            banner = self.pulsar_banner()
        self.write(banner)
        self._more = 0
        
    def interact(self, timeout):
        start = time()
        while time() - start < timeout:
            if self._more:
                prompt = sys.ps2
            else:
                prompt = sys.ps1
            try:
                line = decode_line(self.raw_input(prompt))
            except EOFError:
                self.write("\n")
            else:
                self._more = self.push(line)


class PulsarShell(tasks.CPUboundServer):
    can_kill_arbiter = True
    _app_name = 'pulsarshell'
    console_class = InteractiveConsole
    cfg_apps = ('cpubound',)
    cfg = {'timeout':5,
           'workers':1,
           'loglevel':'none',
           'concurrency':'thread'}
    
    @property
    def console(self):
        return self.local.console
    
    def handler(self):
        imported_objects = {'pshell': self,
                            'pulsar': pulsar,
                            'get_actor': pulsar.get_actor,
                            'spawn': pulsar.spawn,
                            'send': pulsar.send,
                            'Actor': pulsar.Actor}
        try: # Try activating rlcompleter, because it's handy.
            import readline
        except ImportError: #pragma    nocover
            pass
        else: #pragma    nocover
            import rlcompleter
            readline.set_completer(rlcompleter.Completer(imported_objects).complete)
            readline.parse_and_bind("tab:complete")
        self.local.console = self.console_class(imported_objects)
        self.console.setup()
        return self

    def monitor_start(self, monitor):
        monitor.cfg.set('workers', 1)
        monitor.cfg.set('concurrency', 'thread')
        
    def worker_task(self, worker):  #pragma    nocover
        try:
            self.console.interact(0.5*self.cfg.timeout)
        except pulsar.EXIT_EXCEPTIONS:
            worker.send('arbiter', 'stop')
            worker.state = pulsar.ACTOR_STATES.INACTIVE
            




'''Asynchronous HTTP client'''
import pulsar
from pulsar import lib
from pulsar.utils import httpurl

from .iostream import AsyncIOStream


__all__ = ['HttpClient']
    

class HttpConnection(httpurl.HttpConnection):
    
    def connect(self):
        if self.timeout == 0:
            self.sock = AsyncIOStream()
            self.sock.connect((self.host, self.port))
            if self._tunnel_host:
                self._tunnel()
        else:
            httpurl.HttpConnection.connect(self)
            
    @property
    def closed(self):
        if self.timeout == 0:
            if not self.sock.closed:
                return httpurl.is_closed(self.sock.sock)
            else:
                return True
        else:
            return httpurl.is_closed(self.sock)
        

class HttpClient(httpurl.HttpClient):
    timeout = 0
    client_version = pulsar.SERVER_SOFTWARE
    http_connection = HttpConnection
    


'''Classes for testing WSGI servers using the HttpClient'''
from io import BytesIO
import logging
import socket

from pulsar import IStream, create_socket_address
from pulsar.utils.httpurl import HttpClient, HttpRequest, HttpConnectionPool,\
                                    HttpResponse, urlparse, HttpConnection,\
                                    HttpParser
from pulsar.utils.structures import AttributeDictionary
from pulsar.apps.wsgi import server
#from .server import HttpResponse

__all__ = ['HttpTestClient']


class DummyHttpServerConnection(IStream):
    '''This is a simple class simulating a connection on
a Http server. It contains the client response so that the
write method simply write on the client response
object.'''
    def __init__(self, client_response):
        self.client_response = client_response
        self.parser = HttpParser()
        self.server = AttributeDictionary(
                        server_name='local-testing-server',
                        server_port=8888,
                        app_handler=client_response.request.client.wsgi_handler)
        for d in client_response.request_data:
            self.parser.execute(d, len(d))

    def write(self, response):
        for data in response:
            if data:
                self.client_response.parsedata(data)


class TestHttpResponse(HttpResponse):
    request = None
    server_response = None

    def __init__(self, data):
        super(TestHttpResponse, self).__init__(None)
        self.request_data = data

    @property
    def environ(self):
        if self.server_response:
            return self.server_response.environ

    def read(self):
        request = self.request
        if not request:
            raise ValueError('request not available')
        # Create the Dummy test connection
        c = DummyHttpServerConnection(self)
        # Get environment
        environ = server.wsgi_environ(c, c.parser)
        # Create the Server response
        self.server_response = server.HttpResponse(c, environ)
        # Write the response
        c.write(self.server_response)
        return self


class TestHttpConnection(HttpConnection):

    @property
    def data_sent(self):
        if not hasattr(self, '_data_sent'):
            self._data_sent = []
        return self._data_sent

    def send(self, data):
        self.data_sent.append(data)

    def getresponse(self):
        return TestHttpResponse(self.data_sent)


class HttpTestConnectionPool(HttpConnectionPool):

    def get_connection(self):
        return TestHttpConnection(self.host, self.port)

    def release(self, connection):
        pass

    def remove(self, connection):
        pass


class HttpTestClientRequest(HttpRequest):

    def __init__(self, client, url, method, ajax=False, **request):
        super(HttpTestClientRequest, self).__init__(client, url, method,
                                                    **request)
        if ajax:
            self.add_header('x_requested_with', 'XMLHttpRequest')


class HttpTestClient(HttpClient):
    client_version = 'Pulsar-Http-Test-Client'
    request_class = HttpTestClientRequest
    connection_pool = HttpTestConnectionPool

    def __init__(self, test, wsgi_handler, **kwargs):
        self.test = test
        self.wsgi_handler = wsgi_handler
        super(HttpTestClient, self).__init__(**kwargs)



import time
from multiprocessing import Process, current_process
from threading import Thread, current_thread

from pulsar import system, wrap_socket, platform, socket_pair
from pulsar.utils.security import gen_unique_id

from .iostream import AsyncIOStream
from .proxy import ActorProxyMonitor, get_proxy
from .defer import pickle, EXIT_EXCEPTIONS
from .access import get_actor, get_actor_from_id


__all__ = ['Concurrency', 'concurrency']

def concurrency(kind, actor_class, monitor, commands_set, cfg, **params):
    '''Function invoked by the :class:`Arbiter` or a :class:`Monitor` when
spawning a new :class:`Actor`. It created a :class:`Concurrency` instance
which handle the contruction and the lif of an :class:`Actor`.

:paramater kind: Type of concurrency
:paramater monitor: The monitor (or arbiter) managing the :class:`Actor`.
:rtype: a :class:`Councurrency` instance
'''
    if kind == 'monitor':
        c = MonitorConcurrency()
    elif kind == 'thread':
        c = ActorThread()
    elif kind == 'process':
        c = ActorProcess()
    else:
        raise ValueError('Concurrency %s not supported in pulsar' % kind)
    return c.make(kind, actor_class, monitor, commands_set, cfg, **params)


class Concurrency(object):
    '''Actor implementation is responsible for the actual spawning of
actors according to a concurrency implementation. Instances are pickable
and are shared between the :class:`Actor` and its
:class:`ActorProxyMonitor`.

:parameter concurrency: string indicating the concurrency implementation.
    Valid choices are ``monitor``, ``process`` and ``thread``.
:parameter actor_class: :class:`Actor` or one of its subclasses.
:parameter timeout: timeout in seconds for the actor.
:parameter kwargs: additional key-valued arguments to be passed to the actor
    constructor.
'''
    _creation_counter = 0
    address = None
    def make(self, kind, actor_class, monitor, commands_set, cfg, name=None,
             aid=None, **params):
        self.__class__._creation_counter += 1
        self.aid = aid or gen_unique_id()[:8]
        self.age = self.__class__._creation_counter
        self.name = name or actor_class.__name__.lower()
        self.kind = kind
        self.commands_set = commands_set
        self.cfg = cfg
        self.actor_class = actor_class
        self.params = params
        return self.get_actor(monitor)

    @property
    def unique_name(self):
        return '%s(%s)' % (self.name, self.aid)

    def __repr__(self):
        return self.unique_name
    __str__ = __repr__

    def get_actor(self, monitor):
        self.daemon = True
        if monitor.is_arbiter():
            arbiter = monitor
        else:
            arbiter = monitor.arbiter
        self.params['arbiter'] = get_proxy(arbiter)
        self.params['monitor'] = get_proxy(monitor)
        return ActorProxyMonitor(self)
                


class MonitorConcurrency(Concurrency):
    '''An actor implementation for Monitors. Monitors live in the main process
loop and therefore do not require an inbox.'''
    def get_actor(self, arbiter):
        self.params['arbiter'] = arbiter
        return self.actor_class(self)

    def start(self):
        pass

    def is_active(self):
        return self.actor.is_alive()

    @property
    def pid(self):
        return current_process().pid


class ActorConcurrency(Concurrency):

    def run(self):
        try:
            actor = self.actor_class(self)
            actor.start()
        except EXIT_EXCEPTIONS:
            # This is needed in windows in order to avoid useless traceback
            # on KeyboardInterrupt
            pass


class ActorProcess(ActorConcurrency, Process):
    '''Actor on a process'''
    pass


class ActorThread(ActorConcurrency, Thread):
    '''Actor on a thread'''
    def terminate(self):
        '''Called by the main thread to force termination.'''
        actor = get_actor_from_id(self.aid)
        if actor is not None:
            actor.exit()

    @property
    def pid(self):
        return current_process().pid




import sys
from inspect import isclass
import threading

import pulsar
from pulsar import is_failure, async, get_actor
from pulsar.async import commands
from pulsar.async.defer import pickle


__all__ = ['create_test_arbiter',
           'run_on_arbiter',
           'NOT_TEST_METHODS',
           'halt_server',
           'arbiter_test',
           'ActorTestMixin',
           'AsyncAssert']


class MockArbiter(pulsar.Arbiter):
    '''A mock Arbiter for Testing'''
    def _run(self):
        run = super(MockArbiter, self)._run
        self._test_thread = threading.Thread(name='Mock arbiter thread',
                                             target=run)
        self._test_thread.start()
    

NOT_TEST_METHODS = ('setUp', 'tearDown', '_pre_setup', '_post_teardown',
                    'setUpClass', 'tearDownClass')

class TestCallable:

    def __init__(self, test, method_name, istest):
        self.test = test
        self.method_name = method_name
        self.istest = istest
        
    def __repr__(self):
        if isclass(self.test):
            return '%s.%s' % (self.test.__name__, self.method_name)
        else:
            return '%s.%s' % (self.test.__class__.__name__, self.method_name)
    __str__ = __repr__        
    
    @async(max_errors=1, description='Test ')
    def run_test(self, actor):
        test = self.test
        if self.istest:
            test = actor.app.runner.before_test_function_run(test)
        inject_async_assert(test)
        test_function = getattr(test, self.method_name)
        return test_function()
    
    def __call__(self, actor=None):
        actor = actor or get_actor()
        test = self.test
        outcome = self.run_test(actor)
        if self.istest:
            outcome.addBoth(lambda result:
                actor.app.runner.after_test_function_run(test, result))
        return outcome
    

class TestFunction:
    
    def __init__(self, method_name):
        self.method_name = method_name
        self.istest = self.method_name not in NOT_TEST_METHODS
    
    def __repr__(self):
        return self.method_name
    __str__ = __repr__
    
    def __call__(self, test):
        callable = TestCallable(test, self.method_name, self.istest)
        return self.run(callable)
        
    def run(self, callable):
        return callable()
        
        
class TestFunctionOnArbiter(TestFunction):
    
    def run(self, callable):
        actor = get_actor()
        if actor.is_monitor():
            return callable(actor)
        else:
            return actor.send(actor.monitor, 'run', callable)
        
        
def create_test_arbiter(test=True):
    '''Create an instance of MockArbiter for testing'''
    commands_set = set(commands.actor_commands)
    commands_set.update(commands.arbiter_commands)
    arbiter = pulsar.concurrency('monitor', MockArbiter, 1000,
                                 None, 'arbiter', commands_set,
                                 {'__test_arbiter__': test})
    arbiter.start()
    return arbiter
    
def halt_server(exception=None):
    exception = exception or pulsar.HaltServer('testing')
    raise exception
    
def run_on_arbiter(f):
    '''Decorator for running a test function in the arbiter domain. This
can be useful to test Arbiter mechanics.'''
    f.testfunction = TestFunctionOnArbiter(f.__name__)
    return f
    
def arbiter_test(f):
    '''Decorator for testing arbiter mechanics. It creates a mock arbiter
running on a separate thread and run the tet function on the arbiter thread.'''
    d = pulsar.Deferred()
    def work(self):
        yield f(self)
        yield self.arbiter.stop()
    @pulsar.async
    def safe(self):
        yield work(self)
        d.callback(True)
    def _(self):
        self.arbiter = create_test_arbiter()
        while not self.arbiter.started:
            yield pulsar.NOT_DONE
        self.arbiter.ioloop.add_callback(lambda: safe(self))
        yield d
    _.__name__ = f.__name__
    _.__doc__ = f.__doc__
    return _
    
    
class AsyncAssertTest(object):
    
    def __init__(self, a, test, name=None):
        self.a = a
        self.test = test
        self.name = name
        
    def __getattr__(self, name):
        return self.__class__(self.a, self.test, name=name)
    
    def __call__(self, *args):
        d = pulsar.MultiDeferred(args, type=list).lock()
        return d.add_callback(self._check_result)
    
    def _check_result(self, args):
        func = getattr(self.a, self.name, None)
        if func:
            return func(self.test, *args)
        else:
            func = getattr(self.test, self.name)
            return func(*args)
        
        
class AsyncAssert(object):
        
    def __get__(self, instance, instance_type=None):
        return AsyncAssertTest(self, instance)
    
    def assertRaises(self, test, excClass, value):
        def _():
            if is_failure(value):
                value.raise_all()
        test.assertRaises(excClass, _)
        

class ActorTestMixin(object):
    '''A mixin for testing spawning of actors. Make sure this
is the first class you derive from, before the unittest.TestCase, so that
the tearDown method is overwritten.'''
    concurrency = 'thread'
    a = None
    
    @property
    def all_spawned(self):
        if not hasattr(self, '_spawned'):
            self._spawned = []
        return self._spawned
        
    def spawn(self, concurrency=None, **kwargs):
        concurrency = concurrency or self.concurrency
        ad = pulsar.spawn(concurrency=concurrency, **kwargs)
        self.assertTrue(ad.aid)
        self.assertTrue(isinstance(ad, pulsar.ActorProxyDeferred))
        yield ad
        self.a = ad.result
        self.all_spawned.append(self.a)
        self.assertEqual(self.a.aid, ad.aid)
        self.assertTrue(self.a.address)
    
    def stop_actors(self, *args):
        all = args or self.all_spawned
        if len(all) == 1:
            return all[0].stop()
        elif all:
            return MultiDeferred((a.stop() for a in all)).lock()
            
    def tearDown(self):
        return self.stop_actors()
        
        
def inject_async_assert(obj):
    tcls = obj if isclass(obj) else obj.__class__
    if not hasattr(tcls, 'async'):
        tcls.async = AsyncAssert()




import sys
import io
import pickle
from inspect import istraceback

from pulsar import is_failure, CLEAR_ERRORS, make_async, get_actor, async


__all__ = ['TestRequest']


def test_method(cls, method):
    try:
        return cls(method)
    except ValueError:
        return None


class TestRequest(object):
    '''A class which wraps a test case class and runs all its test functions

.. attribute:: testcls

    A :class:`unittest.TestCase` class to be run on this request.

.. attribute:: tag

    A string indicating the tag associated with :attr:`testcls`.
'''
    def __init__(self, testcls, tag):
        self.testcls = testcls
        self.tag = tag

    def __repr__(self):
        return self.testcls.__name__
    __str__ = __repr__

    def start(self, worker):
        '''Run all test functions from the :attr:`testcls` using the
following algorithm:

* Run the class method ``setUpClass`` of :attr:`testcls` if defined.
* Call :meth:`run_test` for each test functions in :attr:`testcls`
* Run the class method ``tearDownClass`` of :attr:`testcls` if defined.
'''
        # Reset the runner
        worker.app.local.pop('runner')
        runner = worker.app.runner
        testcls = self.testcls
        if not isinstance(testcls, type):
            testcls = testcls()
        testcls.tag = self.tag
        testcls.cfg = worker.cfg
        all_tests = runner.loadTestsFromTestCase(testcls)
        run_test_function = runner.run_test_function
        if all_tests.countTestCases():
            skip_tests = getattr(testcls, "__unittest_skip__", False)
            should_stop = False
            test_cls = test_method(testcls, 'setUpClass')
            if test_cls and not skip_tests:
                outcome = run_test_function(testcls,
                                            getattr(testcls,'setUpClass'))
                yield outcome
                should_stop = self.add_failure(test_cls, runner, outcome.result)
            if not should_stop:
                # Loop over all test cases in class
                for test in all_tests:
                    yield self.run_test(test, runner)
            test_cls = test_method(testcls, 'tearDownClass')
            if test_cls and not skip_tests:
                outcome = run_test_function(testcls,getattr(testcls,
                                                        'tearDownClass'))
                yield outcome
                self.add_failure(test_cls, runner, outcome.result)
            # Clear errors
            yield CLEAR_ERRORS
        # send runner result to monitor
        yield worker.send(worker.monitor, 'test_result', testcls.tag,
                          testcls.__name__, runner.result)

    def run_test(self, test, runner):
        '''\
Run a *test* function using the following algorithm

* Run :meth:`_pre_setup` method if available in :attr:`testcls`.
* Run :meth:`setUp` method in :attr:`testcls`.
* Run the test function.
* Run :meth:`tearDown` method in :attr:`testcls`.
* Run :meth:`_post_teardown` method if available in :attr:`testcls`.
'''
        try:
            success = True
            runner.startTest(test)
            run_test_function = runner.run_test_function
            testMethod = getattr(test, test._testMethodName)
            if (getattr(test.__class__, "__unittest_skip__", False) or
                getattr(testMethod, "__unittest_skip__", False)):
                reason = (getattr(test.__class__,
                                  '__unittest_skip_why__', '') or
                          getattr(testMethod,
                                  '__unittest_skip_why__', ''))
                runner.addSkip(test, reason)
                raise StopIteration()
            # _pre_setup function if available
            if hasattr(test,'_pre_setup'):
                outcome = run_test_function(test, test._pre_setup)
                yield outcome
                success = not self.add_failure(test, runner, outcome.result)
            # _setup function if available
            if success:
                outcome = run_test_function(test, test.setUp)
                yield outcome
                if not self.add_failure(test, runner, outcome.result):
                    # Here we perform the actual test
                    outcome = run_test_function(test, testMethod)
                    yield outcome
                    success = not self.add_failure(test, runner, outcome.result)
                    if success:
                        test.result = outcome.result
                    outcome = run_test_function(test, test.tearDown)
                    yield outcome
                    if self.add_failure(test, runner, outcome.result):
                        success = False
                else:
                    success = False
            # _post_teardown
            if hasattr(test,'_post_teardown'):
                outcome = run_test_function(test,test._post_teardown)
                yield outcome
                if self.add_failure(test, runner, outcome.result):
                    success = False
            # run the stopTest
            runner.stopTest(test)
        except StopIteration:
            success = False
        except Exception as e:
            self.add_failure(test, runner, e)
            success = False
        if success:
            runner.addSuccess(test)

    def add_failure(self, test, runner, failure):
        if is_failure(failure):
            for trace in failure:
                e = trace[1]
                try:
                    raise e
                except test.failureException:
                    runner.addFailure(test, trace)
                except:
                    runner.addError(test, trace)
                return True
        else:
            return False


import time
from datetime import timedelta

from pulsar import thread_loop, NOT_DONE
from pulsar.apps import tasks


class RunPyCode(tasks.Job):
    '''execute python code in *code*. There must be a *task_function*
function defined which accept key-valued parameters only.'''
    timeout = timedelta(seconds=60)
    def __call__(self, consumer, code, **kwargs):
        code_local = compile(code, '<string>', 'exec')
        ns = {}
        exec(code_local,ns)
        func = ns['task_function']
        return func(**kwargs)
        

class Addition(tasks.Job):
    timeout = timedelta(seconds=60)
    def __call__(self, consumer, a, b):
        return a+b
    
    
class Asynchronous(tasks.Job):
    
    def __call__(self, consumer, loops=1):
        loop = 0
        rl = thread_loop()
        start = rl.num_loops
        while loop < loops:
            yield NOT_DONE
            loop += 1
        yield {'start': start,
               'end': rl.num_loops,
               'loops': loop}
    
    
class NotOverLap(tasks.Job):
    can_overlap = False
    
    def __call__(self, consumer, lag):
        time.sleep(lag)
        return 'OK'


'''Pulsar HTTP test application::

    python manage.py
'''
import re
import os
import json
import sys
import string
import time
from random import choice
try:
    import pulsar
except ImportError: #pragma    nocover
    sys.path.append('../../')
    import pulsar
    
from pulsar import HttpRedirect, LocalMixin, local_property, version
from pulsar.apps import wsgi, ws
from pulsar.apps.wsgi.server import HttpResponse
from pulsar.utils.structures import OrderedDict
from pulsar.utils.httpurl import Headers, parse_qs, ENCODE_URL_METHODS,\
                                 responses, has_empty_content, addslash,\
                                 itervalues, range, ispy3k, hexmd5, to_bytes,\
                                 WWWAuthenticate
from pulsar.utils.multipart import parse_form_data
from pulsar.utils import events

pyversion = '.'.join(map(str,sys.version_info[:3]))

error_messages = {404: '<p>The requested URL was not found on the server.</p>'}

if ispy3k:  # pragma nocover
    characters = string.ascii_letters + string.digits
else:   # pragma nocover
    characters = string.letters + string.digits

def jsonbytes(data):
    return json.dumps(data, indent=4).encode('utf-8')

def getheaders(environ):
    headers = Headers(kind='client')
    for k in environ:
        if k.startswith('HTTP_'):
            headers[k[5:].replace('_','-')] = environ[k]
    return dict(headers)

def info_data(environ, **params):
    method = environ['REQUEST_METHOD']
    headers = getheaders(environ)
    args = {}
    if method in ENCODE_URL_METHODS:
        qs = environ.get('QUERY_STRING',{})
        if qs:
            args = parse_qs(qs)
    else:
        post, files = parse_form_data(environ)
        args = dict(post)
    data = {'method': method, 'headers': headers, 'args': args}
    data.update(params)
    return jsonbytes(data)


class HttpException(pulsar.HttpException):
    pass


class route(object):
    counter = 0
    def __init__(self, url, method='GET', title='?', params=None):
        cls = self.__class__
        cls.counter += 1
        self.order = cls.counter
        self.method = method.upper()
        self.title = title
        url = addslash(url)
        href = url
        self.params = []
        self.key = tuple((b for b in url.split('/') if b))
        if params:
            if url.endswith('/'):
                url = url[:-1]
            href = url
            for name, value in params:
                url += '/:%s' % name
                href += '/%s' % value
                self.params.append(name)
        self.url = url
        self.href = href

    def render(self):
        return "<li><a href='%s' class='%s'>%s</a> %s.</li>" %\
                 (self.href, self.method.lower(), self.url, self.title)

    def __call__(self, f):
        def _(obj, environ, *args, **kwargs):
            if self.method != environ['REQUEST_METHOD']:
                raise HttpException(status=405)
            else:
                return f(obj, environ, *args, **kwargs)
        _.route = self
        return _


class HttpBin(LocalMixin):
    '''WSGI application running on the server'''
    @local_property
    def routes(self):
        return OrderedDict(((r.route.key, r) for r in sorted(
                                self._routes(), key=lambda x: x.route.order)))

    @classmethod
    def _routes(cls):
        for m in cls.__dict__:
            attr = getattr(cls, m)
            if hasattr(attr, 'route'):
                yield attr

    def __call__(self, environ, start_response):
        response = self.request(environ)
        return response(environ, start_response)

    def request(self, environ):
        try:
            path = environ.get('PATH_INFO')
            if not path or path == '/':
                return self.home(environ)
            else:
                path = path[1:]
                leaf = True
                if path.endswith('/'):
                    leaf = False
                    path = path[:-1]
                keys = tuple(path.split('/'))
                bits = []
                route = None
                while keys:
                    route = self.routes.get(keys)
                    if route:
                        break
                    bits.insert(0, keys[-1])
                    keys = keys[:-1]
                if route is None:
                    raise HttpException(status=404)
                return route(self, environ, bits)
        except HttpException as e:
            status = e.status
            if has_empty_content(status, environ['REQUEST_METHOD']):
                return wsgi.WsgiResponse(status, response_headers=e.headers)
            else:
                content = error_messages.get(status,'')
                title = responses.get(status)
                content = '<h1>%s - %s</h1>%s' % (status, title, content)
                return self.render(content, title=title, status=status,
                                   headers=e.headers)

    def load(self, name):
        name = os.path.join(os.path.dirname(os.path.abspath(__file__)),name)
        with open(name,'r') as file:
            return file.read()

    def redirect(self, location='/'):
        raise HttpRedirect(location)

    def response(self, data, status=200, content_type=None, headers=None):
        content_type = content_type or 'application/json'
        return wsgi.WsgiResponse(status,
                                 content=data,
                                 content_type=content_type,
                                 response_headers=headers)

    def render(self, body, title=None, status=200, headers=None):
        template = self.load('template.html')
        title = title or 'Pulsar HttpBin'
        html = (template % (title, version, body, pyversion)).encode('utf-8')
        return self.response(html, status=status, content_type='text/html',
                             headers=headers)

    # ROUTE FUNCTIONS

    def home(self, environ):
        data = '<ul>\n%s\n</ul>' % '\n'.join(
                            (r.route.render() for r in itervalues(self.routes)))
        return self.render(data)

    @route('get', title='Returns GET data')
    def request_get(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        return self.response(info_data(environ))
    
    @route('getsize', title='Returns a preset size of data',
           params=[('size', int(2.5*HttpResponse.MAX_CHUNK))])
    def request_getsize(self, environ, bits):
        if len(bits) == 1:
             size = int(bits[0])
        else:
            raise HttpException(status=404)
        data = {'size': size,
                'data': ''.join(('d' for n in range(size)))}
        return self.response(jsonbytes(data))

    @route('post', method='POST', title='Returns POST data')
    def request_post(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        return self.response(info_data(environ))

    @route('put', method='PUT', title='Returns PUT data')
    def request_put(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        return self.response(info_data(environ))
    
    @route('patch', method='PATCH', title='Returns PATCH data')
    def request_patch(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        return self.response(info_data(environ))
    
    @route('delete', method='DELETE', title='Returns DELETE data')
    def request_delete(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        return self.response(info_data(environ))

    @route('redirect', title='302 Redirect n times', params=[('n', 6)])
    def request_redirect(self, environ, bits):
        if bits:
            if len(bits) > 2:
                raise HttpException(status=404)
            num = int(bits[0])
        else:
            num = 1
        num -= 1
        if num > 0:
            return self.redirect('/redirect/%s' % num)
        else:
            return self.redirect('/get')

    @route('gzip', title='Returns gzip encoded data')
    def request_gzip(self, environ, bits):
        if bits:
            raise HttpException(status=404)
        data = self.response(info_data(environ, gzipped=True))
        middleware = wsgi.middleware.GZipMiddleware(10)
        # Apply the gzip middleware
        middleware(environ, data)
        return data

    @route('cookies', title='Returns cookie data')
    def request_cookies(self, environ, bits):
        cookies = {'cookies': environ['HTTP_COOKIE']}
        return self.response(jsonbytes(cookies))

    @route('cookies/set', 'GET', 'Sets a simple cookie',
           params=(('name', 'package'), ('value', 'pulsar')))
    def request_cookies_set(self, environ, bits):
        if len(bits) == 2:
            key = bits[0]
            value = bits[1]
            if key and value:
                response = wsgi.WsgiResponse(302,
                                    response_headers=[('location','/cookies')])
                response.set_cookie(key, value=value)
                return response
        else:
            raise HttpException(status=404)
        cookies = {'cookies': environ['HTTP_COOKIE']}
        return self.response(jsonbytes(cookies))

    @route('status','GET', 'Returns given HTTP Status code',
           params=[('status', 418)])
    def request_status(self, environ, bits):
        try:
            number = int(bits[0]) if len(bits) == 1 else 404
        except:
            raise HttpException(status=404)
        raise HttpException(status=number)

    @route('response-headers', title='Returns response headers')
    def request_response_headers(self, environ, bits):
        if bits:
            raise HttpException(status=404)

        class Gen:
            headers = None
            def __call__(self, headers=None, **kwargs):
                self.headers = headers
            def generate(self):
                #yield a byte so that headers are sent
                yield '{'
                # we must have the headers now
                headers = jsonbytes(dict(self.headers))
                yield headers[1:]
        gen = Gen()
        events.bind('http-headers', gen)
        data = wsgi.WsgiResponse(
                            200,
                            content=gen.generate(),
                            content_type='application/json')
        return data

    @route('basic-auth', title='Challenges HTTPBasic Auth',
           params=(('username','username'),('password','password')))
    def request_challenge_auth(self, environ, bits):
        if len(bits) == 2:
            auth = environ.get('HTTP_AUTHORIZATION')
            if auth and auth.type == 'basic':
                if auth.authenticated(environ, *bits):
                    data = jsonbytes({'autheinticated': True,
                                      'username': auth.username})
                    return self.response(data)
            h = ('WWW-Authenticate', str(WWWAuthenticate.basic("Fake Realm")))
            raise HttpException(status=401, headers=[h])
        else:
            raise HttpException(status=404)
        
    @route('digest-auth', title='Challenges HTTP Digest Auth',
           params=(('qop', 'auth'),('username','username'),
                   ('password','password')))
    def request_challenge_digest_auth(self, environ, bits):
        if len(bits) == 3:
            auth = environ.get('HTTP_AUTHORIZATION')
            if auth and auth.authenticated(environ, *bits[1:]):
                data = jsonbytes({'autheinticated': True,
                                  'username': auth.username})
                return self.response(data)
            nonce = hexmd5(to_bytes('%d' % time.time()) + os.urandom(10))
            digest = WWWAuthenticate.digest("Fake Realm", nonce,
                                            opaque=hexmd5(os.urandom(10)),
                                            qop=bits[:1])
            raise HttpException(status=401,
                                headers=[('WWW-Authenticate', str(digest))])
        else:
            raise HttpException(status=404)
        
    @route('stream', title='Stream m chunk of data n times',
           params=(('m', 300), ('n', 20)))
    def request_stream(self, environ, bits):
        if len(bits) == 2:
            try:
                m = int(bits[0])
                n = int(bits[1])
            except:
                raise HttpException(status=404)
        else:
            raise HttpException(status=404)
        stream = ('Chunk %s\n%s\n\n' % (i+1, ''.join((choice(characters) for\
                    _ in range(m)))) for i in range(n))
        return self.response(stream, content_type='text/plain')


class handle(ws.WS):

    def on_message(self, msg):
        path = self.path
        if path == '/echo':
            return msg
        elif path == '/streaming':
            return json.dumps([(i,random()) for i in range(100)])


def server(description=None, **kwargs):
    description = description or 'Pulsar HttpBin'
    app = wsgi.WsgiHandler(middleware=(wsgi.clean_path_middleware,
                                       wsgi.cookies_middleware,
                                       wsgi.authorization_middleware,
                                       HttpBin(),))
    return wsgi.WSGIServer(app, description=description, **kwargs)


if __name__ == '__main__':  #pragma    nocover
    server().start()



import os
import sys
import json
import textwrap
import logging
import time
from datetime import datetime, timedelta
from functools import partial, reduce
from email.utils import formatdate

import pulsar
from pulsar import maybe_async, is_async, is_failure, log_failure, NOT_DONE
from pulsar.utils.httpurl import Headers, SimpleCookie, responses,\
                                 has_empty_content, string_type, ispy3k,\
                                 to_bytes, REDIRECT_CODES, iteritems

if ispy3k:  #thanks to @chrismcdonoug
    from urllib.parse import unquote_to_bytes
    unquote_to_bytes_wsgi = lambda bs: unquote_to_bytes(bs).decode('latin-1')
else:   #pragma    nocover
    from urlparse import unquote as unquote_to_bytes_wsgi

__all__ = ['WsgiHandler',
           'WsgiResponse',
           'WsgiResponseGenerator',
           'wsgi_iterator',
           'handle_wsgi_error',
           'wsgi_error_msg',
           'is_streamed']


LOGGER = logging.getLogger('pulsar.wsgi')

def is_streamed(content):
    try:
        len(content)
    except TypeError:
        return True
    return False

def wsgi_iterator(gen, encoding=None):
    encoding = encoding or 'utf-8'
    for data in gen:
        data = maybe_async(data)
        while is_async(data):
            yield b''
            data = maybe_async(data)
        if data is NOT_DONE:
            yield b''
        elif data is None:
            continue
        elif is_failure(data):
            log_failure(data)
        else:
            if isinstance(data, bytes):
                yield data
            elif isinstance(data, string_type):
                yield data.encode(encoding)
            else:
                for b in wsgi_iterator(data, encoding):
                    yield b

def cookie_date(epoch_seconds=None):
    """Formats the time to ensure compatibility with Netscape's cookie
    standard.

    Accepts a floating point number expressed in seconds since the epoch in, a
    datetime object or a timetuple.  All times in UTC.  The :func:`parse_date`
    function can be used to parse such a date.

    Outputs a string in the format ``Wdy, DD-Mon-YYYY HH:MM:SS GMT``.

    :param expires: If provided that date is used, otherwise the current.
    """
    rfcdate = formatdate(epoch_seconds)
    return '%s-%s-%s GMT' % (rfcdate[:7], rfcdate[8:11], rfcdate[12:25])

def set_cookie(cookies, key, value='', max_age=None, expires=None, path='/',
                domain=None, secure=False, httponly=False):
    '''Set a cookie key into the cookies dictionary *cookies*.'''
    cookies[key] = value
    if expires is not None:
        if isinstance(expires, datetime):
            delta = expires - expires.utcnow()
            # Add one second so the date matches exactly (a fraction of
            # time gets lost between converting to a timedelta and
            # then the date string).
            delta = delta + timedelta(seconds=1)
            # Just set max_age - the max_age logic will set expires.
            expires = None
            max_age = max(0, delta.days * 86400 + delta.seconds)
        else:
            cookies[key]['expires'] = expires
    if max_age is not None:
        cookies[key]['max-age'] = max_age
        # IE requires expires, so set it if hasn't been already.
        if not expires:
            cookies[key]['expires'] = cookie_date(time.time() + max_age)
    if path is not None:
        cookies[key]['path'] = path
    if domain is not None:
        cookies[key]['domain'] = domain
    if secure:
        cookies[key]['secure'] = True
    if httponly:
        cookies[key]['httponly'] = True
        

class WsgiResponseGenerator(object):

    def __init__(self, environ, start_response):
        self.environ = environ
        self.start_response = start_response
        self.middleware = []

    def __iter__(self):
        raise NotImplementedError()

    def start(self, response):
        '''Start the response generator'''
        response.middleware.extend(self.middleware)
        for b in response(self.environ, self.start_response):
            yield b


class WsgiResponse(WsgiResponseGenerator):
    '''A WSGI response wrapper initialized by a WSGI request middleware.
Instances are callable using the standard WSGI call::

    response = WsgiResponse(200)
    response(environ, start_response)

A :class:`WsgiResponse` is an iterable over bytes to send back to the requesting
client.

.. attribute:: status_code

    Integer indicating the HTTP status, (i.e. 200)

.. attribute:: response

    String indicating the HTTP status (i.e. 'OK')

.. attribute:: status

    String indicating the HTTP status code and response (i.e. '200 OK')

.. attribute:: environ

    The dictionary of WSGI environment if passed to the constructor.

'''
    _started = False
    DEFAULT_STATUS_CODE = 200
    def __init__(self, status=None, content=None, response_headers=None,
                 content_type=None, encoding=None, environ=None,
                 start_response=None):
        super(WsgiResponse, self).__init__(environ, start_response)
        self.status_code = status or self.DEFAULT_STATUS_CODE
        self.encoding = encoding
        self.cookies = SimpleCookie()
        self.headers = Headers(response_headers, kind='server')
        self.content = content
        if content_type is not None:
            self.content_type = content_type

    @property
    def started(self):
        return self._started

    @property
    def path(self):
        if self.environ:
            return self.environ.get('PATH_INFO','')

    @property
    def method(self):
        if self.environ:
            return self.environ.get('REQUEST_METHOD')

    @property
    def connection(self):
        if self.environ:
            return self.environ.get('pulsar.connection')

    def _get_content(self):
        return self._content
    def _set_content(self, content):
        if not self._started:
            if content is None:
                content = ()
            elif ispy3k: #what a fucking pain
                if isinstance(content, str):
                    content = bytes(content, 'latin-1')
            else: #pragma    nocover
                if isinstance(content, unicode):
                    content = bytes(content, 'latin-1')
            if isinstance(content, bytes):
                content = (content,)
            self._content = content
        else:
            raise RuntimeError('Cannot set content. Already iterated')
    content = property(_get_content, _set_content)
    
    def _get_content_type(self):
        return self.headers.get('content-type')
    def _set_content_type(self, typ):
        if typ:
            self.headers['content-type'] = typ
        else:
            self.headers.pop('content-type', None)
    content_type = property(_get_content_type, _set_content_type)

    def __call__(self, environ, start_response, exc_info=None):
        '''Make sure the headers are set.'''
        if not exc_info:
            for rm in self.middleware:
                try:
                    rm(environ, self)
                except:
                    LOGGER.error('Exception in response middleware',
                                 exc_info=True)
        start_response(self.status, self.get_headers(), exc_info=exc_info)
        return self
    
    def start(self):
        self.__call__(self.environ, self.start_response)

    def length(self):
        if not self.is_streamed:
            return reduce(lambda x,y: x+len(y), self.content, 0)

    @property
    def response(self):
        return responses.get(self.status_code)

    @property
    def status(self):
        return '%s %s' % (self.status_code, self.response)

    def __str__(self):
        return self.status

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self)

    @property
    def is_streamed(self):
        """If the response is streamed (the response is not an iterable with
length information) this property is `True`.  In this case streamed
means that there is no information about the number of iterations.
This is usually `True` if a generator is passed to the response object."""
        return is_streamed(self.content)

    def __iter__(self):
        if self._started:
            raise RuntimeError('WsgiResponse can be iterated only once')
        self._started = True
        if self.is_streamed:
            return wsgi_iterator(self.content, self.encoding)
        else:
            return iter(self.content)

    def __len__(self):
        return len(self.content)

    def set_cookie(self, key, **kwargs):
        """
        Sets a cookie.

        ``expires`` can be a string in the correct format or a
        ``datetime.datetime`` object in UTC. If ``expires`` is a datetime
        object then ``max_age`` will be calculated.
        """
        set_cookie(self.cookies, key, **kwargs)

    def delete_cookie(self, key, path='/', domain=None):
        set_cookie(self.cookies, key, max_age=0, path=path, domain=domain,
                   expires='Thu, 01-Jan-1970 00:00:00 GMT')

    def get_headers(self):
        headers = self.headers
        if has_empty_content(self.status_code, self.method):
            headers.pop('content-type', None)
            headers.pop('content-length', None)
        elif not self.is_streamed:
            cl = 0
            for c in self.content:
                cl += len(c)
            headers['Content-Length'] = str(cl)
        for c in self.cookies.values():
            headers['Set-Cookie'] = c.OutputString()
        return list(headers)


class WsgiHandler(object):
    '''An handler for application conforming to python WSGI_.

.. attribute:: middleware

    List of callable WSGI middleware callable which accept
    ``environ`` and ``start_response`` as arguments.
    The order matter, since the response returned by the callable
    is the non ``None`` value returned by a middleware.

.. attribute:: response_middleware

    List of functions of the form::

        def ..(environ, start_response, response):
            ...

    where ``response`` is the first not ``None`` value returned by
    the middleware.

'''
    def __init__(self, middleware=None, response_middleware=None, **kwargs):
        if middleware:
            middleware = list(middleware)
        self.middleware = middleware or []
        self.response_middleware = response_middleware or []

    def __call__(self, environ, start_response):
        '''The WSGI callable'''
        response = None
        for middleware in self.middleware:
            response = middleware(environ, start_response)
            if response is not None:
                break
        if response is None:
            raise pulsar.Http404(environ.get('PATH_INFO','/'))
        if hasattr(response, 'middleware'):
            response.middleware.extend(self.response_middleware)
        return response

    def get(self, route='/'):
        '''Fetch a middleware with the given *route*. If it is not found
 return ``None``.'''
        for m in self.middleware:
            if getattr(m,'route',None) == route:
                return m


error_messages = {
    500: 'An exception has occurred while evaluating your request.',
    404: 'Cannot find what you are looking for.'
}

def wsgi_error_msg(response, msg):
    if response.content_type == 'application/json':
        return json.dumps({'status': response.status_code,
                           'message': msg})
    else:
        return msg
    
    
class dump_environ(object):
    
    def __init__(self, environ):
        self.environ = environ
        
    def __str__(self):
        env = iteritems(self.environ)
        return '\n%s\n' % '\n'.join(('%s = %s' % (k, v) for k, v in env))
        
        
def handle_wsgi_error(environ, trace=None, content_type=None,
                      encoding=None):
    '''The default handler for errors while serving an Http requests.

:parameter environ: The WSGI environment.
:parameter trace: the error traceback. If not avaiable it will be obtained from
    ``sys.exc_info()``.
:parameter content_type: Optional content type.
:parameter encoding: Optional charset.
:return: a :class:`WsgiResponse`
'''
    content_type=content_type or environ.get('CONTENT_TYPE')
    response = WsgiResponse(content_type=content_type,
                            environ=environ,
                            encoding=encoding)
    if not trace:
        trace = sys.exc_info()
    error = trace[1]
    content = None
    response.status_code = getattr(error, 'status', 500)
    response.headers.update(getattr(error, 'headers', None) or ())
    path = ' @ path "%s"' % environ.get('PATH_INFO','/')
    e = dump_environ(environ)
    if response.status_code == 500:
        LOGGER.critical('Unhandled exception during WSGI response %s.%s',
                        path, e, exc_info=trace)
    else:
        LOGGER.info('WSGI %s status code %s.%s',
                    response.status_code, path, e, exc_info=trace)
    if has_empty_content(response.status_code) or\
       response.status_code in REDIRECT_CODES:
        content = ()
        response.content_type = None
    else:
        renderer = environ.get('wsgi_error_handler')
        if renderer:
            try:
                content = renderer(environ, response, trace)
                if is_failure(content):
                    content.log()
                    content = None
            except:
                LOGGER.critical('Error while rendering error', exc_info=True)
                content = None
    if content is None:
        msg = error_messages.get(response.status_code) or response.response
        if response.content_type == 'text/html':
            content = textwrap.dedent("""\
            <!DOCTYPE html>
            <html>
              <head>
                <title>{0[reason]}</title>
              </head>
              <body>
                <h1>{0[reason]}</h1>
                {0[msg]}
                <h3>{0[version]}</h3>
              </body>
            </html>
            """).format({"reason": response.status, "msg": msg,
                         "version": pulsar.SERVER_SOFTWARE})
        else:
            content = wsgi_error_msg(response, msg)
    response.content = content
    return response


#
# Required by Cython to build Hiredis extensions
#
import os
import sys
from distutils.core import setup
from distutils.extension import Extension
from distutils.errors import (CCompilerError, DistutilsExecError,
                              DistutilsPlatformError)
try:
    from Cython.Distutils import build_ext
    cython_message = None
except ImportError:
    from distutils.command.build_ext import build_ext
    cython_message = 'Cannot build C extensions, Cython is not installed.'

ext_errors = (CCompilerError, DistutilsExecError, DistutilsPlatformError) 
if sys.platform == 'win32' and sys.version_info > (2, 6):
   # 2.6's distutils.msvc9compiler can raise an IOError when failing to
   # find the compiler
   ext_errors += (IOError,)

class BuildFailed(Exception):

    def __init__(self, msg=None):
        if not msg:
            msg = str(sys.exc_info()[1])
        self.msg = msg


class tolerant_build_ext(build_ext):
    # This class allows C extension building to fail. From SQLAlchemy

    def run(self):
        if cython_message:
            raise BuildFailed(cython_message) 
        try:
            build_ext.run(self)
        except DistutilsPlatformError:
            raise BuildFailed()

    def build_extension(self, ext):
        if cython_message:
            raise BuildFailed(cython_message)
        try:
            build_ext.build_extension(self, ext)
        except ext_errors:
            raise BuildFailed()
        except ValueError:
            # this can happen on Windows 64 bit, see Python issue 7511
            if "'path'" in str(sys.exc_info()[1]): # works with both py 2/3
                raise BuildFailed()
            raise

################################################################################
##    EXTENSIONS
ext_modules  = Extension('pulsar.lib._pulsar', ['lib/http-parser/http_parser.c',
                                                'lib/src/parser.pyx'])


base_path = os.path.split(os.path.abspath(__file__))[0]
include_dirs = [os.path.join(base_path,'http-parser'),
                os.path.join(base_path,'src')]

libparams = {
             'ext_modules': [ext_modules],
             'cmdclass': {'build_ext' : tolerant_build_ext},
             'include_dirs': include_dirs
             }


'''A deferred module with almost the same API as twisted.'''
import sys
from copy import copy
import logging
import traceback
from threading import current_thread, local
from collections import deque, namedtuple
from inspect import isgenerator, isfunction, ismethod, istraceback
from time import sleep

from pulsar import AlreadyCalledError, DeferredFailure, HaltServer

from .access import thread_loop

EXIT_EXCEPTIONS = (KeyboardInterrupt, SystemExit, HaltServer)

__all__ = ['Deferred',
           'MultiDeferred',
           'DeferredGenerator',
           'Failure',
           'as_failure',
           'is_failure',
           'raise_failure',
           'log_failure',
           'is_async',
           'maybe_async',
           'make_async',
           'safe_async',
           'async',
           'multi_async',
           'ispy3k',
           'NOT_DONE',
           'STOP_ON_FAILURE',
           'EXIT_EXCEPTIONS',
           'CLEAR_ERRORS']

ispy3k = sys.version_info >= (3, 0)
if ispy3k:
    import pickle
    iteritems = lambda d : d.items()
    itervalues = lambda d : d.values()
    def raise_error_trace(err, traceback):
        if istraceback(traceback):
            raise err.with_traceback(traceback)
        else:
            raise err
    range = range
    can_generate = lambda gen: hasattr(gen, '__next__')
else:   # pragma : nocover
    import cPickle as pickle
    from pulsar.utils._py2 import *
    iteritems = lambda d : d.iteritems()
    itervalues = lambda d : d.itervalues()
    range = xrange
    can_generate = lambda gen: hasattr(gen, 'next')

# Special objects
class NOT_DONE(object):
    pass

class STOP_ON_FAILURE(object):
    pass

class CLEAR_ERRORS(object):
    pass

EMPTY_DICT = {}

logger = logging.getLogger('pulsar.async.defer')

remote_stacktrace = namedtuple('remote_stacktrace', 'error_class error trace')

pass_through = lambda result: result

def iterdata(stream, start=0):
    '''Iterate over a stream which is either a dictionary or a list. This
iterator is over key-value pairs for a dictionary, and index-value pairs
for a list.'''
    if isinstance(stream, dict):
        return iteritems(stream)
    else:
        return enumerate(stream, start)

def is_generalised_generator(value):
    '''Check if *value* is a generator. This is more general than the
inspect.isgenerator function.'''
    return hasattr(value, '__iter__') and not hasattr(value, '__len__')

def is_stack_trace(trace):
    if isinstance(trace, remote_stacktrace):
        return True
    elif isinstance(trace,tuple) and len(trace) == 3:
        return istraceback(trace[2]) or\
                 (trace[2] is None and isinstance(trace[1],trace[0]))
    return False

def is_failure(value):
    return isinstance(value, Failure)

def as_failure(value, msg=None):
    '''Convert *value* into a :class:`Failure` if it is a stack trace or an
exception, otherwise returns *value*.'''
    if isinstance(value, Exception):
        exc_info = sys.exc_info()
        if value == exc_info[1]:
            return Failure(exc_info, msg)
        else:
            return Failure((value.__class__, value, None), msg)
    elif is_stack_trace(value):
        return Failure(value, msg)
    else:
        return value

def is_async(obj):
    '''Check if *obj* is an asynchronous instance'''
    return isinstance(obj, Deferred)

def maybe_async(val, description=None, max_errors=None):
    '''Convert *val* into an asynchronous instance only if *val* is a generator
or a function. If *val* is a :class:`Deferred` it checks if it has been
called and all callbacks have been consumed.
In this case it returns the :attr:`Deferred.result` attribute.'''
    if isgenerator(val):
        val = DeferredGenerator(val, max_errors=max_errors,
                                description=description)
    if isfunction(val) or ismethod(val):
        try:
            val = maybe_async(val())
        except:
            val = sys.exc_info()
    if is_async(val):
        return val.result_or_self()
    else:
        return as_failure(val)

def make_async(val=None, description=None, max_errors=None):
    '''Convert *val* into an :class:`Deferred` asynchronous instance
so that callbacks can be attached to it.

:parameter val: can be a generator or any other value. If a generator, a
    :class:`DeferredGenerator` instance will be returned.
:parameter max_errors: the maximum number of errors tolerated if *val* is
    a generator. Default `None`.
:return: a :class:`Deferred` instance.

This function is useful when someone needs to treat a value as a deferred::

    v = ...
    make_async(v).add_callback(...)

'''
    val = maybe_async(val, description, max_errors)
    if not is_async(val):
        d = Deferred(description=description)
        d.callback(val)
        return d
    else:
        return val
    
def safe_async(f, args=None, kwargs=None, description=None, max_errors=None):
    '''Execute function *f* safely and **always** returns an asynchronous
result.

:parameter f: function to execute
:parameter args: tuple of positional arguments for *f*.
:parameter kwargs: dictionary of key-word parameters for *f*.
:parameter description: Optional description for the :class:`Deferred` returned.
:parameter max_errors: the maximum number of errors tolerated if a :class:`DeferredGenerator`
    is returned.
:return: a :class:`Deferred` instance.
'''
    try:
        kwargs = kwargs if kwargs is not None else EMPTY_DICT
        args = args or ()
        result = f(*args, **kwargs)
    except:
        result = sys.exc_info()
    return make_async(result, max_errors=max_errors, description=description)

def log_failure(failure):
    '''Log the *failure* if *failure* is a :class:`Failure`.'''
    if is_failure(failure):
        failure.log()
    return failure

############################################################### DECORATORS
class async:
    '''A decorator class which transforms a function into
an asynchronous callable.
    
:parameter max_errors: The maximum number of errors permitted if the
    asynchronous value is a :class:`DeferredGenerator`.
:parameter description: optional description.

Typical usage::

    @async()
    def myfunction(...):
        ...
'''
    def __init__(self, max_errors=None, description=None):
         self.max_errors = max_errors
         self.description = description or 'async decorator for '

    def __call__(self, func):
        description = '%s%s' % (self.description, func.__name__)
        def _(*args, **kwargs):
            return safe_async(func, args=args, kwargs=kwargs,
                              max_errors=self.max_errors,
                              description=description)
        _.__name__ = func.__name__
        _.__doc__ = func.__doc__
        return _

def multi_async(func):
    '''Decorator for a function *func* which returns an iterable over, possibly
asynchronous, values. This decorator create an instance of a
:class:`MultiDeferred` called once all asynchronous values have been caled.'''
    def _(*args, **kwargs):
        try:
            return MultiDeferred(func(*args, **kwargs), type=list).lock()
        except Exception as e:
            return make_async(e)
    _.__name__ = func.__name__
    _.__doc__ = func.__doc__
    return _

def raise_failure(f):
    '''Decorator for raising failures'''
    def _(*args, **kwargs):
        r = f(*args, **kwargs)
        if is_failure(r):
            r.raise_all()
        return r
    _.__name__ = f.__name__
    _.__doc__ = f.__doc__
    return _

############################################################### FAILURE
class Failure(object):
    '''Aggregate failures during :class:`Deferred` callbacks.

.. attribute:: traces

    List of (``errorType``, ``errvalue``, ``traceback``) occured during
    the execution of a :class:`Deferred`.

'''
    logged = False
    def __init__(self, err=None, msg=None):
        self.should_stop = False
        self.msg = msg or ''
        self.traces = []
        self.append(err)

    def __repr__(self):
        return '\n\n'.join(self.format_all())
    __str__ = __repr__

    def append(self, trace):
        '''Add new failure to self.'''
        if trace:
            if isinstance(trace, Failure):
                self.traces.extend(trace.traces)
            elif isinstance(trace, Exception):
                self.traces.append(sys.exc_info())
            elif is_stack_trace(trace):
                self.traces.append(trace)
        return self

    def clear(self):
        self.traces = []

    def format_all(self):
        for exctype, value, tb in self:
            if istraceback(tb):
                tb = traceback.format_exception(exctype, value, tb)
            if tb:
                yield '\n'.join(tb)
            else:
                yield str(value)

    def __getstate__(self):
        traces = []
        for exctype, value, tb in self:
            if istraceback(tb):
                tb = traceback.format_exception(exctype, value, tb)
            traces.append(remote_stacktrace(exctype, value, tb))
        state = self.__dict__.copy()
        state['traces'] = traces
        return state

    def __getitem__(self, index):
        return self.traces[index]

    def __len__(self):
        return len(self.traces)

    def __iter__(self):
        return iter(self.traces)

    def raise_all(self, first=True):
        pos = 0 if first else -1
        if self.traces and isinstance(self.traces[pos][1],Exception):
            eclass, error, trace = self.traces.pop()
            self.log()
            raise_error_trace(error, trace)
        else:
            self.log()
            N = len(self.traces)
            if N == 1:
                raise DeferredFailure(
                    'There was one failure during callbacks.')
            elif N > 1:
                raise DeferredFailure(
                    'There were {0} failures during callbacks.'.format(N))

    @property
    def trace(self):
        if self.traces:
            return self.traces[-1]
        else:
            return (None,None,None)

    def log(self, log=None):
        if not self.logged:
            self.logged = True
            log = log or logger
            for e in self:
                log.critical(self.msg, exc_info=e)

############################################################### Deferred
class Deferred(object):
    """The main class of the pulsar asynchronous tools.
It is a callback which will be put off until later.
The implementation is very similar to the ``twisted.defer.Deferred`` object.

.. attribute:: called

    ``True`` if the deferred was called. In this case the asynchronous result
    is ready and available in the :attr:`result`.

.. attribute:: running

    ``True`` if the deferred is running callbacks.
    
.. attribute:: paused

    Integer indicating the number of times this :class:`Deferred` has been
    paused because the result of a callback was another :class::`Deferred`.

.. attribute:: result

    This is available once the :class:`Deferred` has been called back. Note,
    this can be anything, including another :class:`Deferred`. Trying to access
    this attribute when :attr:`called` is ``False`` will result in an
    ``AttributeError`` exception.
"""
    paused = 0
    _called = False
    _runningCallbacks = False

    def __init__(self, description=None):
        self._description = description
        self._callbacks = deque()

    def __repr__(self):
        v = self._description or self.__class__.__name__
        if self.called:
            v += ' (called)'
        return v

    def __str__(self):
        return self. __repr__()

    @property
    def called(self):
        return self._called

    @property
    def running(self):
        return self._runningCallbacks

    def add_callback(self, callback, errback=None):
        """Add a callback as a callable function.
The function takes at most one argument, the result passed to the
:meth:`callback` method. If the *errback* callable is provided it will
be called when an exception occurs."""
        errback = errback if errback is not None else pass_through
        if hasattr(callback, '__call__') and hasattr(errback, '__call__'):
            self._callbacks.append((callback, errback))
            self._run_callbacks()
        else:
            raise TypeError('callback must be callable')
        return self

    def add_errback(self, errback):
        '''Same as :meth:`add_callback` but only for errors.'''
        return self.add_callback(pass_through, errback)

    def addBoth(self, callback):
        '''Equivalent to `self.add_callback(callback, callback)`.'''
        return self.add_callback(callback, callback)

    def add_callback_args(self, callback, *args, **kwargs):
        return self.add_callback(\
                lambda result : callback(result, *args, **kwargs))

    def callback(self, result=None):
        '''Run registered callbacks with the given *result*.
This can only be run once. Later calls to this will raise
:class:`AlreadyCalledError`. If further callbacks are added after
this point, :meth:`add_callback` will run the *callbacks* immediately.

:return: the *result* input parameter
'''
        if isinstance(result, Deferred):
            raise RuntimeError('Received a deferred instance from '
                               'callback function')
        elif self._called:
            raise AlreadyCalledError('Deferred %s already called' % self)
        self.result = as_failure(result)
        self._called = True
        self._run_callbacks()
        return self.result

    def result_or_self(self):
        '''It returns the :attr:`result` only if available and all
callbacks have been consumed, otherwise it returns this :class:`Deferred`.
Users should use this method to obtain the result, rather than accessing
directly the :attr:`result` attribute.'''
        return self.result if self._called and not self._callbacks else self

    ##################################################    INTERNAL METHODS
    def _run_callbacks(self):
        if not self._called or self._runningCallbacks or self.paused:
            return
        while self._callbacks:
            callbacks = self._callbacks.popleft()
            callback = callbacks[is_failure(self.result)]
            try:
                self._runningCallbacks = True
                try:
                    self.result = maybe_async(callback(self.result))
                finally:
                    self._runningCallbacks = False
            except Exception as e:
                self._add_exception(e)
            else:
                if isinstance(self.result, Deferred):
                    # Add a pause
                    self._pause()
                    # Add a callback to the result to resume callbacks
                    self.result.addBoth(self._continue)
                    break

    def _pause(self):
        """Stop processing until :meth:`unpause` is called."""
        self.paused += 1

    def _unpause(self):
        """Process all callbacks made since :meth:`pause` was called."""
        self.paused -= 1
        self._run_callbacks()

    def _continue(self, result):
        self.result = result
        self._unpause()
        return self.result

    def _add_exception(self, e):
        if not isinstance(self.result, Failure):
            self.result = Failure(e)
        else:
            self.result.append(e)


class DeferredGenerator(Deferred):
    '''A :class:`Deferred` for a generator over, possibly, deferred objects.
The callback will occur once the generator has stopped
(when it raises StopIteration), or a preset maximum number of errors has
occurred.

:parameter gen: a generator.
:parameter max_errors: The maximum number of exceptions allowed before
    stopping the generator and raise exceptions. By default the
    generator will continue regardless of errors, accumulating them into
    the final result.
'''
    def __init__(self, gen, max_errors=None, description=None):
        self.gen = gen
        if not can_generate(gen):
            raise TypeError('DeferredGenerator requires an iterable')
        self.max_errors = max(1, max_errors) if max_errors else 0
        self._consumed = 0
        self.errors = Failure()
        super(DeferredGenerator,self).__init__(description=description)
        self.loop = thread_loop()
        self._consume()

    def _resume_in_thread(self, result=None):
        # When the generator finds an asynchronous object still waiting
        # for results, it adds this callback to resume the generator at the
        # next iteration in the eventloop.
        if self.loop.tid != current_thread().ident:
            # Generators are not thread-safe. If the callback is on a different
            # thread we restart the generator on the original thread
            # by adding a callback in the generator eventloop.
            self.loop.add_callback(lambda: self._consume(result))
        else:
            self._consume(result)
        # IMPORTANT! We return the original result.
        # Otherwise we just keep adding deferred objects to the callbacks.
        return result

    def _consume(self, last_result=None):
        '''override the deferred consume private method for handling the
generator. Important! Callbacks are always added to the event loop on the
current thread.'''
        if isinstance(last_result, Failure):
            if self.should_stop(last_result):
                return self.conclude()
        try:
            result = next(self.gen)
            self._consumed += 1
        except EXIT_EXCEPTIONS:
            raise
        except StopIteration as e:
            # The generator has finished producing data
            return self.conclude(last_result)
        except Exception as e:
            if self.should_stop(e):
                return self.conclude()
            return self._consume()
        else:
            if result == NOT_DONE:
                # The NOT_DONE object indicates that the generator needs to
                # abort so that the event loop can continue. This generator
                # will resume at the next event loop.
                self.loop.add_callback(self._consume, wake=False)
                return self
            result = maybe_async(result)
            if is_async(result):
                # The result is asynchronous and it is not ready yet.
                # We pause the generator and attach a callback to continue
                # on the same thread.
                return result.addBoth(self._resume_in_thread)
            if result == CLEAR_ERRORS:
                self.errors.clear()
                result = None
            # continue with the loop
            return self._consume(result)

    def should_stop(self, failure):
        self.errors.append(failure)
        return self.max_errors and len(self.errors) >= self.max_errors

    def conclude(self, last_result=None):
        # Conclude the generator and callback the listeners
        result = last_result if not self.errors else self.errors
        self.gen = None
        self.errors = None
        #log_failure(result)
        return self.callback(result)


############################################################### MultiDeferred
class MultiDeferred(Deferred):
    '''A :class:`Deferred` for managing a stream if independent objects
which may be :class:`Deferred`.

.. attribute:: lock

    If ``True`` items can no longer be added to this :class:`MultiDeferred`.
    
.. attribute:: type

    The type of multideferred. Either a ``list`` or a ``dict``.
'''
    _locked = False

    def __init__(self, data=None, type=None, fireOnOneErrback=False,
                 handle_value=None):
        self._deferred = {}
        self._failures = Failure()
        self.fireOnOneErrback = fireOnOneErrback
        self.handle_value = handle_value
        if not type:
            type = data.__class__ if data is not None else list
        if not issubclass(type, (list, dict)):
            type = list
        self._stream = type()
        super(MultiDeferred, self).__init__()
        if data:
            self.update(data)

    @property
    def locked(self):
        return self._locked
        
    @property
    def type(self):
        return self._stream.__class__.__name__

    def lock(self):
        '''Lock the :class:`MultiDeferred` so that no new items can be added.
If it was alread :attr:`locked` a runtime exception is raised.'''
        if self._locked:
            raise RuntimeError(self.__class__.__name__ +\
                        ' cannot be locked twice.')
        self._locked = True
        if not self._deferred:
            self._finish()
        return self

    def update(self, stream):
        '''Update the :class:`MultiDeferred` with new data. It works for
both ``list`` and ``dict`` types.'''
        add = self._add
        try:
            for key, value in iterdata(stream, len(self._stream)):
                add(key, value)
        except:
            raise
        return self

    def append(self, value):
        '''Append only works for a list type multideferred'''
        if self.type == 'list':
            self._add(len(self._stream), value)
        else:
            raise RuntimeError('Cannot append a value to a "dict" type '
                               'multideferred')

    def _add(self, key, value):
        if self._locked:
            raise RuntimeError(self.__class__.__name__ +\
                               ' cannot add a dependent once locked.')
        if is_generalised_generator(value):
            value = list(value)
        value = maybe_async(value)
        if isinstance(value, (dict, list, tuple, set, frozenset)):
            value = self._make(value)    
        if not is_async(value) and self.handle_value:
            try:
                val = self.handle_value(value)
            except Exception as e:
                value = as_failure(e)
            else:
                if val is not value:
                    return self._add(key, val)
        self._setitem(key, value)
        # add callback if an asynchronous value
        if is_async(value):
            self._add_deferred(key, value)

    def _make(self, value):
        md = self.__class__(value, fireOnOneErrback=self.fireOnOneErrback,
                            handle_value=self.handle_value)
        return maybe_async(md.lock())

    def _add_deferred(self, key, value):
        self._deferred[key] = value
        value.addBoth(lambda result: self._deferred_done(key, result))

    def _deferred_done(self, key, result):
        self._deferred.pop(key, None)
        self._setitem(key, result)
        if self._locked and not self._deferred and not self.called:
            self._finish()
        return result

    def _finish(self):
        if not self._locked:
            raise RuntimeError(self.__class__.__name__ +\
                               ' cannot finish until completed.')
        if self._deferred:
            raise RuntimeError(self.__class__.__name__ +\
                               ' cannot finish whilst waiting for '
                               'dependents %r' % self._deferred)
        if self.fireOnOneErrback and self._failures:
            self.callback(self._failures)
        else:
            self.callback(self._stream)

    def _setitem(self, key, value):
        stream = self._stream
        if isinstance(stream, list) and key == len(stream):
            stream.append(value)
        else:
            stream[key] = value
        if is_failure(value):
            self._failures.append(value)




class PulsarException(Exception):
    '''Base class of all Pulsar Exception'''

class Timeout(PulsarException):
    '''Raised when a timeout occurs'''
    def __init__(self, msg, timeout=None):
        self.timeout = timeout
        if timeout:
            msg = msg + ' Timeout {0} surpassed.'.format(self.timeout)
        super(Timeout,self).__init__(msg)

class ImproperlyConfigured(PulsarException):
    '''A :class:`PulsarException` raised when pulsar has inconsistent
configuration.'''
    pass

class PulsarInternetException(PulsarException):
    '''base class of all Pulsar Internet Exception'''
    pass

class MailboxError(PulsarException):
    pass

class ActorAlreadyStarted(PulsarException):
    '''A :class:`PulsarException` raised when trying to start an actor already started'''

class HaltServer(PulsarInternetException):

    def __init__(self, reason='Halt', signal=None):
        super(HaltServer,self).__init__(reason)
        self.signal = signal


class CommandError(PulsarException):
    '''Exception raised when executing a Command'''
    pass


class CommandNotFound(PulsarException):

    def __init__(self, name):
        super(CommandNotFound, self).__init__(
                            'Command "%s" not available' % name)

class AuthenticationError(PulsarException):
    pass


class ConnectionError(PulsarInternetException):
    pass


class CouldNotParse(PulsarInternetException):
    pass


class DeferredFailure(PulsarException):
    pass


class AlreadyCalledError(PulsarException):
    '''Raised when a :class:`Deferred` instance receives more than
one :meth:`Deferred.callback`.'''


class CannotCallBackError(PulsarException):
    pass


class AlreadyRegistered(PulsarException):
    pass

class NotRegisteredWithServer(PulsarException):
    pass


##################################################################### HTTP
class HttpException(Exception):
    status = 500
    def __init__(self, msg='', status=None, handler=None, strict=False,
                 headers=None):
        self.status = status or self.status
        self.handler = handler
        self.strict = strict
        self.headers = headers
        super(HttpException,self).__init__(msg)


class HttpRedirect(HttpException):
    status = 302
    def __init__(self, location, status=None):
        super(HttpRedirect, self).__init__(status=status or self.status,
              headers=[('location', location)])


class PermissionDenied(HttpException):
    status = 403


class Http404(HttpException):
    status = 404

    
class MethodNotAllowed(HttpException):
    status = 405
    


import threading
from threading import Thread, current_thread
from multiprocessing import current_process

__all__ = ['thread_loop',
           'thread_ioloop',
           'get_actor',
           'set_local_data',
           'is_mainthread',
           'PulsarThread',
           'process_local_data',
           'thread_local_data']

def is_mainthread(thread=None):
    '''Check if thread is the main thread. If *thread* is not supplied check
the current thread'''
    thread = thread if thread is not None else current_thread() 
    return isinstance(thread, threading._MainThread)

def process_local_data(name=None):
    '''Fetch the current process local data dictionary. If *name* is not
``None`` it returns the value at *name*, otherwise it return the process data
dictionary.'''
    ct = current_process()
    if not hasattr(ct, '_pulsar_local'):
        ct._pulsar_local = ProcessLocal()
    loc = ct._pulsar_local
    if name:
        return getattr(loc, name, None)
    else:
        return loc
            
def thread_local_data(name, value=None):
    '''Set or retrieve an attribute *name* from the curren thread. If *value*
is None, it will get the value otherwise it will set the value.'''
    ct = current_thread()
    if is_mainthread(ct):
        loc = process_local_data()
    elif not hasattr(ct, '_pulsar_local'):
        ct._pulsar_local = threading.local()
        loc = ct._pulsar_local
    else:
        loc = ct._pulsar_local
    if value is not None:
        if hasattr(loc, name):
            if getattr(loc, name) is not value:
                raise RuntimeError(
                            '%s is already available on this thread' % name)
        else:
            setattr(loc, name, value)
    return getattr(loc, name, None)

def thread_loop(ioloop=None):
    '''Returns the event loop (:class:`IOLoop`) on the current thread
if available.'''
    return thread_local_data('eventloop', ioloop)

def thread_ioloop(ioloop=None):
    '''Returns the :class:`IOLoop` on the current thread if available.'''
    return thread_local_data('ioloop', ioloop)

get_actor = lambda: thread_local_data('actor')

def set_actor(actor):
    '''Returns the actor running the current thread.'''
    actor = thread_local_data('actor', value=actor)
    if actor.impl.kind == 'thread':
        process_local_data('thread_actors')[actor.aid] = actor
    return actor

def remove_actor(actor):
    '''Remove actor from threaded_actors dictionary'''
    if actor.impl.kind == 'thread':
        process_local_data('thread_actors').pop(actor.aid, None)

def get_actor_from_id(aid):
    '''Retrieve an actor from its actor id. This function can be used by
actors with thread concurrency ince they live in the arbiter process domain.'''
    actors = process_local_data('thread_actors')
    if actors:
        return actors.get(aid)

def set_local_data(actor):
    set_actor(actor)
    thread_loop(actor.requestloop)
    thread_ioloop(actor.ioloop)
    
    
class PulsarThread(Thread):
    
    def __init__(self, *args, **kwargs):
        self.actor = get_actor()
        super(PulsarThread, self).__init__(*args, **kwargs)
        
    def run(self):
        set_local_data(self.actor)
        super(PulsarThread, self).run()
        
        
class plocal(object):
    pass
        
class ProcessLocal(object):
    def __init__(self):
        self.thread_actors = {}
        
    def local(self):
        return plocal()


'''Tests the "taskqueue" example.'''
from time import time, sleep
from datetime import datetime, timedelta

from pulsar import send, get_application, get_actor, NOT_DONE
from pulsar.apps import tasks, rpc
from pulsar.utils.timeutils import timedelta_seconds
from pulsar.apps.test import unittest, run_on_arbiter, dont_run_with_thread

from .manage import server


CODE_TEST = '''\
import time
def task_function(N = 10, lag = 0.1):
    time.sleep(lag)
    return N*N
'''

class TestTaskClasses(unittest.TestCase):
    
    def testTask(self):
        self.assertRaises(NotImplementedError, tasks.Task.get_task, None, 1)
        self.assertRaises(NotImplementedError, tasks.Task.save_task, None, 1)
        self.assertRaises(NotImplementedError, tasks.Task.delete_tasks, None, 1)
        task = tasks.Task()
        self.assertFalse(task.on_received())
        self.assertFalse(task.on_start())
        self.assertFalse(task.on_timeout())
        self.assertFalse(task.on_finish())
        

class TestTaskQueueOnThread(unittest.TestCase):
    concurrency = 'thread'
    app = None
    
    @classmethod
    def name_tq(cls):
        return 'testtask_'+cls.concurrency
    
    @classmethod
    def name_rpc(cls):
        return cls.name_tq() + '_rpc'
    
    @classmethod
    def setUpClass(cls):
        # The name of the task queue application
        s = server(cls.name_tq(), bind='127.0.0.1:0',
                   concurrency=cls.concurrency)
        outcome = send('arbiter', 'run', s)
        yield outcome
        app = outcome.result
        cls.app = app
        uri = 'http://{0}:{1}'.format(*cls.app.address)
        cls.proxy = rpc.JsonProxy(uri)

    @classmethod
    def tearDownClass(cls):
        if cls.app:
            yield send('arbiter', 'kill_actor', cls.name_tq())
            yield send('arbiter', 'kill_actor', cls.name_rpc())

    @run_on_arbiter
    def testMeta(self):
        '''Tests meta attributes of taskqueue'''
        app = get_application(self.name_tq())
        self.assertTrue(app)
        self.assertEqual(app.name, self.name_tq())
        self.assertTrue(app.registry)
        scheduler = app.scheduler
        self.assertTrue(scheduler.entries)
        job = app.registry['runpycode']
        self.assertEqual(job.type,'regular')
        self.assertTrue(job.can_overlap)
        id = job.make_task_id((),{})
        self.assertTrue(id)
        self.assertNotEqual(id,job.make_task_id((),{}))
        
    @run_on_arbiter
    def testRegistry(self):
        app = get_application(self.name_tq())
        self.assertTrue(isinstance(app.registry, dict))
        regular = app.registry.regular()
        periodic = app.registry.periodic()
        self.assertTrue(regular)
        self.assertTrue(periodic)
        
    @run_on_arbiter
    def testRpcMeta(self):
        app = get_application(self.name_rpc())
        self.assertTrue(app)
        self.assertEqual(app.name, self.name_rpc())
        rpc = app.callable
        root = rpc.handler
        tq = root.taskqueue
        self.assertEqual(tq, self.name_tq())

    @run_on_arbiter
    def testCheckNextRun(self):
        app = get_application(self.name_tq())
        scheduler = app.scheduler
        scheduler.tick(app)
        self.assertTrue(scheduler.next_run > datetime.now())
        
    @run_on_arbiter
    def testNotOverlap(self):
        app = get_application(self.name_tq())
        self.assertTrue('notoverlap' in app.registry)
        r1 = app.scheduler.queue_task(app.monitor, 'notoverlap', (1,), {})
        self.assertEqual(str(r1), 'notoverlap(%s)' % r1.id)
        self.assertTrue(r1._queued)
        r2 = app.scheduler.queue_task(app.monitor, 'notoverlap', (1,), {})
        self.assertFalse(r2._queued)
        id = r1.id
        self.assertEqual(id, r2.id)
        # We need to make sure the first task is completed
        get_task = app.scheduler.get_task
        while get_task(id).status in tasks.UNREADY_STATES:
            yield NOT_DONE
        self.assertEqual(get_task(id).status, tasks.SUCCESS)
    
    @run_on_arbiter    
    def testIdNotOverlap(self):
        '''Check `make_task_id` when `can_overlap` attribute is set to False.'''
        app = get_application(self.name_tq())
        job = app.registry['notoverlap']
        self.assertEqual(job.type, 'regular')
        self.assertFalse(job.can_overlap)
        #
        id = job.make_task_id((),{})
        self.assertTrue(id)
        self.assertEqual(id, job.make_task_id((),{}))
        #
        id = job.make_task_id((10,'bla'),{'p':45})
        self.assertTrue(id)
        self.assertEqual(id,job.make_task_id((10,'bla'),{'p':45}))
        #
        id = job.make_task_id((),{'p':45,'c':'bla'})
        self.assertTrue(id)
        self.assertEqual(id,job.make_task_id((),{'p':45,'c':'bla'}))
        self.assertNotEqual(id,job.make_task_id((),{'p':45,'d':'bla'}))
        self.assertNotEqual(id,job.make_task_id((),{'p':45,'c':'blas'}))
        
    @run_on_arbiter
    def testDeleteTask(self):
        app = get_application(self.name_tq())
        r1 = app.scheduler.queue_task(app.monitor, 'addition', (1,4), {})
        id = r1.id
        get_task = app.scheduler.get_task
        while get_task(id).status in tasks.UNREADY_STATES:
            yield NOT_DONE
        r2 = get_task(id)
        self.assertEqual(r1.id, r2.id)
        r2 = get_task(id, remove=True)
        self.assertEqual(get_task(id), None)
        self.assertEqual(get_task(r1), r1)
        app.scheduler.delete_tasks()
        
    def test_rpc_ping(self):
        self.assertEqual(self.proxy.ping(), 'pong')
        
    def test_rpc_job_list(self):
        jobs = self.proxy.job_list()
        self.assertTrue(jobs)
        self.assertTrue(isinstance(jobs, list))
        d = dict(jobs)
        pycode = d['runpycode']
        self.assertEqual(pycode['type'], 'regular')
        jobs = self.proxy.job_list(jobnames=['runpycode'])
        self.assertEqual(len(jobs), 1)
        jobs = self.proxy.job_list(jobnames=['xxxxxx'])
        self.assertEqual(len(jobs), 0)
        
    def test_rpc_next_scheduled_tasks(self):
        next = self.proxy.next_scheduled_tasks()
        self.assertTrue(next)
        self.assertEqual(len(next), 2)
        next = self.proxy.next_scheduled_tasks(jobnames=['testperiodic'])
        self.assertTrue(next)
        self.assertEqual(len(next), 2)
        self.assertEqual(next[0], 'testperiodic')
        self.assertTrue(next[1] >= 0)
        
    def test_run_new_task_Error(self):
        self.assertRaises(rpc.InvalidParams, self.proxy.run_new_task)
        self.assertRaises(rpc.InvalidParams, self.proxy.run_new_task,
                          jobname='xxxx', bla='foo')
        
    def test_run_new_task_RunPyCode(self):
        '''Run a new task from the *runpycode* task factory.'''
        r = self.proxy.run_new_task(jobname='runpycode', code=CODE_TEST, N=3)
        self.assertTrue(r)
        self.assertTrue(r['id'])
        self.assertTrue(r['time_executed'])
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertEqual(r['result'], 9)
        
    def test_run_new_task_addition(self):
        r = self.proxy.run_new_task(jobname='addition', a=40, b=50)
        self.assertTrue(r)
        self.assertTrue(r['time_executed'])
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertEqual(r['result'], 90)
        
    def test_run_new_task_periodicerror(self):
        r = self.proxy.run_new_task(jobname='testperiodicerror')
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.FAILURE)
        self.assertTrue('kaputt' in r['result'])
        
    def test_run_new_task_asynchronous(self):
        r = self.proxy.run_new_task(jobname='asynchronous', loops=3)
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.SUCCESS)
        result = r['result']
        self.assertEqual(result['loops'], 3)
        self.assertEqual(result['end']-result['start'], 3)
        
    def test_run_new_task_expiry(self):
        r = self.proxy.run_new_task(jobname='addition', a=40, b=50,
                                    meta_data={'expiry': time()})
        self.assertTrue(r)
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.REVOKED)
        
    def test_run_producerconsumer(self):
        '''A task which produce other tasks'''
        sample = 10
        r = self.proxy.run_new_task(jobname='standarddeviation',
                                    sample=sample, size=100)
        self.assertTrue(r)
        while r['status'] in tasks.UNREADY_STATES:
            yield NOT_DONE
            r = self.proxy.get_task(id=r['id'])
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertTrue(tasks.nice_task_message(r))
        # We check for the tasks created
        stasks = []
        while len(stasks) < sample:
            ts = self.proxy.get_tasks(from_task=r['id'])
            stasks = []
            for t in ts:
                if t['status'] not in tasks.UNREADY_STATES:
                    stasks.append(t)
                    self.assertEqual(t['status'], tasks.SUCCESS)
        self.assertEqual(len(stasks), sample)
        

#class TestTaskRpc(unittest.TestCase):
class TestTaskRpc(object):
    '''Test the Rpc and Taskqueue server, including rpc commands
in the TaskQueueRpcMixin class'''
    concurrency = 'process'
    timeout = 3

    def testTaskQueueLink(self):
        '''Check the task_queue_manager in the rpc handler.'''
        app = self.app
        self.assertEqual(app.name,self._name_rpc)
        callable = app.callable
        self.assertTrue(callable.handler.task_queue_manager)
        task_queue_manager = callable.handler.task_queue_manager
        self.assertEqual(task_queue_manager.name,self._name)

    def testKillTaskWorker(self):
        r = self.p.server_info()
        m = dict(((m['name'],m) for m in r['monitors']))
        tq = m[self._name]
        for worker in tq['workers']:
            aid = worker['aid']
            r = self.p.kill_actor(aid)
            self.assertTrue(r)


@dont_run_with_thread
class TestTaskQueueOnProcess(TestTaskQueueOnThread):
    concurrency = 'process'


# -*- coding: utf-8 -
#
# This file is part of gunicorn released under the MIT license.
# See the NOTICE for more information.

from optparse import make_option

from django.core.management.base import BaseCommand, CommandError

from gunicorn.app.djangoapp import DjangoApplicationCommand
from gunicorn.config import make_settings


def make_options():
    g_settings = make_settings(ignore=("version"))

    keys = g_settings.keys()

    def sorter(k):
        return (g_settings[k].section, g_settings[k].order)

    opts = [
        make_option('--adminmedia', dest='admin_media_path', default='',
        help='Specifies the directory from which to serve admin media.')
    ]

    for k in keys:
        if k in ('pythonpath', 'django_settings',):
            continue

        setting = g_settings[k]
        if not setting.cli:
            continue

        args = tuple(setting.cli)

        kwargs = {
            "dest": setting.name,
            "metavar": setting.meta or None,
            "action": setting.action or "store",
            "type": setting.type or "string",
            "default": None,
            "help": "%s [%s]" % (setting.short, setting.default)
        }
        if kwargs["action"] != "store":
            kwargs.pop("type")

        opts.append(make_option(*args, **kwargs))

    return tuple(opts)

GUNICORN_OPTIONS = make_options()


class Command(BaseCommand):
    option_list = BaseCommand.option_list + GUNICORN_OPTIONS
    help = "Starts a fully-functional Web server using pulsar."
    args = '[optional port number, or ipaddr:port or unix:/path/to/sockfile]'

    # Validation is called explicitly each time the server is reloaded.
    requires_model_validation = False

    def handle(self, addrport=None, *args, **options):
        if args:
            raise CommandError('Usage is run_gunicorn %s' % self.args)

        if addrport:
            options['bind'] = addrport

        admin_media_path = options.pop('admin_media_path', '')
        DjangoApplicationCommand(options, admin_media_path).run()


from .httpurl import native_str

class _lazy:

    def __init__(self, f, args, kwargs):
        self._value = None
        self._f = f
        self.args = args
        self.kwargs = kwargs
    
    def __str__(self):
        if self._value is None:
            self._value = native_str(self._f(*self.args, **self.kwargs) or '')
        return self._value
    __repr__ = __str__


def lazy_string(f):
    def _(*args, **kwargs):
        return _lazy(f, args, kwargs)
    return _
    


'''\
An asynchronous task-queue built on top :class:`pulsar.Application` framework.
By creating :class:`Job` classes in a similar way you can do for celery_,
this application gives you all you need for running them with very
little setup effort::

    from pulsar.apps import tasks

    tq = tasks.TaskQueue(tasks_path='path.to.tasks.*')
    tq.start()

.. _tasks-actions:

Tutorial
==============

Actions
~~~~~~~~~~~~~~~

The :class:`Taskqueue` application adds the following
:ref:`remote actions <api-remote_commands>` to its workers:

* **addtask** to add a new task to the task queue::

    send(taskqueue, 'addtask', jobname, task_extra, *args, **kwargs)

 * *jobname*: the name of the :class:`Job` to run.
 * *task_extra*: dictionary of extra parameters to pass to the :class:`Task`
   constructor. Usually a empty dictionary.
 * *args*: positional arguments for the :ref:`job callable <job-callable>`.
 * *kwargs*: key-valued arguments for the :ref:`job callable <job-callable>`.

* **addtask_noack** same as **addtask** but without acknowleding the sender::

    send(taskqueue, 'addtask_noack', jobname, task_extra, *args, **kwargs)
    
* **get_task** retrieve task information. This can be already executed or not.
  The implementation is left to the :meth:`Task.get_task` method::
  
    send(taskqueue, 'get_task', id)
    
* **get_tasks** retrieve information for tasks which satisfy the filtering.
  The implementation is left to the :meth:`Task.get_tasks` method::
  
    send(taskqueue, 'get_tasks', **filters)
  
    
Jobs
~~~~~~~~~~~~~~~~

An application implements several :class:`Job`
classes which specify the way each :class:`Task` is run.
Each job class is a task-factory, therefore, a task is always associated
with one job, which can be of two types:

* standard (:class:`Job`)
* periodic (:class:`PeriodicJob`)

.. _job-callable:

To define a job is simple, subclass from :class:`Job` and implement the
**job callable method**::

    from pulsar.apps import tasks

    class Addition(tasks.Job):

        def __call__(self, consumer, a, b):
            "Add two numbers"
            return a+b
            
    class Sampler(tasks.Job):

        def __call__(self, consumer, sample, size=10):
            ...

The *consumer*, instance of :class:`TaskConsumer`, is passed by the
:class:`TaskQueue` and should always be the first positional argument in the
callable function.
The remaining positional arguments and/or key-valued parameters are needed by
your job implementation.

Task Class
~~~~~~~~~~~~~~~~~

By default, tasks are constructed using an in-memory implementation of
:class:`Task`. To use a different implementation, for example one that
saves tasks on a database, subclass :class:`Task` and pass the new class
to the :class:`TaskQueue` constructor::

    from pulsar.apps import tasks

    class TaskDatabase(tasks.Task):

        def on_created(self):
            return save2db(self)

        def on_received(self):
            return save2db(self)

        def on_start(self):
            return save2db(self)

        def on_finish(self):
            return save2db(self)

        @classmethod
        def get_task(cls, id, remove = False):
            return taskfromdb(id)


    tq = tasks.TaskQueue(task_class=TaskDatabase, tasks_path='path.to.tasks.*')
    tq.start()


.. _tasks-callbacks:

Task callbacks
~~~~~~~~~~~~~~~~~~~

When creating your own :class:`Task` class all you need to override are the four
task callbacks:

* :meth:`Task.on_created` called by the taskqueue when it creates a new task
  instance.
* :meth:`Task.on_received` called by a worker when it receives the task.
* :meth:`Task.on_start` called by a worker when it starts the task.
* :meth:`Task.on_finish` called by a worker when it ends the task.


and :meth:`Task.get_task` classmethod for retrieving tasks instances.

.. _task-state:

Task states
~~~~~~~~~~~~~

A :class:`Task` can have one of the following :attr:`Task.status` string:

* ``PENDING`` A task waiting for execution and unknown.
* ``RETRY`` A task is retrying calculation.
* ``RECEIVED`` when the task is received by the task queue.
* ``STARTED`` task execution has started.
* ``REVOKED`` the task execution has been revoked. One possible reason could be
  the task has timed out.
* ``UNKNOWN`` task execution is unknown.
* ``FAILURE`` task execution has finished with failure.
* ``SUCCESS`` task execution has finished with success.


.. attribute:: FULL_RUN_STATES

    The set of states for which a :class:`Task` has run:
    ``FAILURE`` and ``SUCCESS``

.. attribute:: READY_STATES

    The set of states for which a :class:`Task` has finished:
    ``REVOKED``, ``FAILURE`` and ``SUCCESS``


Queue
~~~~~~~~~~~~~~

By default the queue is implemented using the multiprocessing.Queue
from the standard python library. To specify a different queue you can
use the ``task-queue`` flag from the command line::

    python myserverscript.py --task-queue dotted.path.to.callable

or by setting the ``task_queue_factory`` parameter in the config file
or in the :class:`TaskQueue` constructor.


.. _celery: http://celeryproject.org/
'''
import os
from datetime import datetime

import pulsar
from pulsar import to_string, safe_async
from pulsar.utils.importer import import_modules, module_attribute

from .exceptions import *
from .task import *
from .models import *
from .scheduler import Scheduler
from .states import *
from .rpc import *


class TaskQueueFactory(pulsar.Setting):
    app = 'cpubound'
    name = "task_queue_factory"
    section = "Task Consumer"
    flags = ["-q", "--task-queue"]
    default = "pulsar.Queue"
    desc = """The task queue factory to use."""

    def get(self):
        return module_attribute(self.value)


class TaskSetting(pulsar.Setting):
    virtual = True
    app = 'tasks'


class TaskPath(TaskSetting):
    name = "tasks_path"
    section = "Task Consumer"
    meta = "STRING"
    validator = pulsar.validate_list
    cli = ["--tasks-path"]
    default = ['pulsar.apps.tasks.testing']
    desc = """\
        List of python dotted paths where tasks are located.
        """


class CPUboundServer(pulsar.Application):
    '''A CPU-bound application server.'''
    _app_name = 'cpubound'

    def get_ioqueue(self):
        '''Return the distributed task queue which produces tasks to
be consumed by the workers.'''
        return self.cfg.task_queue_factory()

    def request_instance(self, worker, fd, request):
        return request

    def on_event(self, worker, fd, request):
        request = self.request_instance(worker, fd, request)
        if request is not None:
            c = self.local.current_requests
            if c is None:
                c = []
                self.local.current_requests = c
            c.append(request)
            yield safe_async(request.start, args=(worker,))
            try:
                c.remove(request)
            except ValueError:
                pass

#################################################    TASKQUEUE COMMANDS
taskqueue_cmnds = set()

@pulsar.command(internal=True, commands_set=taskqueue_cmnds)
def addtask(client, actor, caller, jobname, task_extra, *args, **kwargs):
    kwargs.pop('ack', None)
    return actor.app._addtask(actor, caller, jobname, task_extra, True,
                              args, kwargs)

@pulsar.command(internal=True, ack=False, commands_set=taskqueue_cmnds)
def addtask_noack(client, actor, caller, jobname, task_extra, *args, **kwargs):
    kwargs.pop('ack', None)
    return actor.app._addtask(actor, caller, jobname, task_extra, False,
                              args, kwargs)

@pulsar.command(internal=True, commands_set=taskqueue_cmnds)
def save_task(client, actor, caller, task):
    #import time
    #time.sleep(0.1)
    return actor.app.scheduler.save_task(task)

@pulsar.command(internal=True, commands_set=taskqueue_cmnds)
def delete_tasks(client, actor, caller, ids):
    return actor.app.scheduler.delete_tasks(ids)

@pulsar.command(commands_set=taskqueue_cmnds)
def get_task(client, actor, id):
    return actor.app.scheduler.get_task(id)

@pulsar.command(commands_set=taskqueue_cmnds)
def get_tasks(client, actor, **parameters):
    return actor.app.scheduler.get_tasks(**parameters)

@pulsar.command(commands_set=taskqueue_cmnds)
def job_list(client, actor, jobnames=None):
    return list(actor.app.job_list(jobnames=jobnames))

@pulsar.command(commands_set=taskqueue_cmnds)
def next_scheduled(client, actor, jobnames=None):
    return actor.app.scheduler.next_scheduled(jobnames=jobnames)

@pulsar.command(commands_set=taskqueue_cmnds)
def wait_for_task(client, actor, id, timeout=3600):
    # wait for a task to finish for at most timeout seconds
    scheduler = actor.app.scheduler
    return scheduler.task_class.wait_for_task(scheduler, id, timeout)


class TaskQueue(CPUboundServer):
    '''A :class:`pulsar.CPUboundServer` for consuming
tasks and managing scheduling of tasks.

.. attribute:: registry

    Instance of a :class:`JobRegistry` containing all
    registered :class:`Job` instances.
'''
    _app_name = 'tasks'
    cfg_apps = ('cpubound',)
    cfg = {'timeout': '3600', 'backlog': 1}
    commands_set = taskqueue_cmnds
    task_class = TaskInMemory
    '''The :class:`Task` class for storing information about task execution.

Default: :class:`TaskInMemory`
'''
    scheduler_class = Scheduler
    '''The scheduler class. Default: :class:`Scheduler`.'''

    @property
    def scheduler(self):
        '''A :class:`Scheduler` which send task to the task queue and
produces of periodic tasks according to their schedule of execution.

At every event loop, the :class:`pulsar.ApplicationMonitor` running
the :class:`TaskQueue` application, invokes the :meth:`Scheduler.tick`
which check for tasks to be scheduled.

Check the :meth:`TaskQueue.monitor_task` callback
for implementation.'''
        return self.local.scheduler

    def request_instance(self, worker, fd, request):
        return self.scheduler.get_task(request)

    def monitor_task(self, monitor):
        '''Override the :meth:`pulsar.Application.monitor_task` callback
to check if the scheduler needs to perform a new run.'''
        s = self.scheduler
        if s:
            if s.next_run <= datetime.now():
                s.tick(monitor)

    def handler(self):
        # Load the application callable, the task consumer
        if self.callable:
            self.callable()
        import_modules(self.cfg.tasks_path)
        self.local.scheduler = Scheduler(self)
        return self

    def monitor_handler(self):
        return self.handler()

    def job_list(self, jobnames=None):
        return self.scheduler.job_list(jobnames=jobnames)

    @property
    def registry(self):
        global registry
        return registry

    # Internals
    def _addtask(self, monitor, caller, jobname, task_extra, ack, args, kwargs):
        task = self.scheduler.queue_task(monitor, jobname, args, kwargs,
                                         **task_extra)
        if ack:
            return task



'''Tests the websocket middleware in pulsar.apps.ws.'''
from random import randint
import struct

from pulsar import send
from pulsar.apps.ws import Frame, WebSocket, int2bytes, i2b,\
                            WebSocketProtocolError, FrameParser, HttpClient
from pulsar.apps.test import unittest, dont_run_with_thread

from .manage import server


class TestWebSocketThread(unittest.TestCase):
    app = None
    concurrency = 'thread'
    
    @classmethod
    def setUpClass(cls):
        s = server(bind='127.0.0.1:0', name=cls.__name__,
                   concurrency=cls.concurrency)
        outcome = send('arbiter', 'run', s)
        yield outcome
        cls.app = outcome.result
        cls.uri = 'http://{0}:{1}'.format(*cls.app.address)
        cls.ws_uri = 'ws://{0}:{1}/data'.format(*cls.app.address)
        cls.ws_echo = 'ws://{0}:{1}/echo'.format(*cls.app.address)
        
    @classmethod
    def tearDownClass(cls):
        if cls.app is not None:
            yield send('arbiter', 'kill_actor', cls.app.name)
    
    def testHyBiKey(self):
        w = WebSocket(None)
        v = w.challenge_response('dGhlIHNhbXBsZSBub25jZQ==')
        self.assertEqual(v, "s3pPLMBiTxaQ9kYGzzhZRbK+xOo=")
        
    def testBadRequests(self):
        c = HttpClient()
        outcome = c.post(self.ws_uri)
        yield outcome
        response = outcome.result
        self.assertEqual(response.status_code, 400)
        #
        outcome = c.get(self.ws_uri, headers=[('Sec-Websocket-Key', '')])
        yield outcome
        response = outcome.result
        self.assertEqual(response.status_code, 400)
        #
        outcome = c.get(self.ws_uri, headers=[('Sec-Websocket-Key', 'bla')])
        yield outcome
        response = outcome.result
        self.assertEqual(response.status_code, 400)
        #
        outcome = c.get(self.ws_uri, headers=[('Sec-Websocket-version', 'xxx')])
        yield outcome
        response = outcome.result
        self.assertEqual(response.status_code, 400)
    
    def testUpgrade(self):
        c = HttpClient()
        outcome = c.get(self.ws_echo)
        yield outcome
        ws = outcome.result
        response = ws.handshake 
        self.assertEqual(response.status_code, 101)
        self.assertEqual(response.headers['upgrade'], 'websocket')
        # Send a message to the websocket
        outcome = ws.execute('Hi there!')
        yield outcome
        response = outcome.result
        self.assertEqual(response.body, 'Hi there!')
        self.assertFalse(response.masking_key)
        

@dont_run_with_thread
class TestWebSocketProcess(TestWebSocketThread):
    concurrency = 'process'
    
    
class FrameTest(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        cls.bdata = b''.join((i2b(randint(0,255)) for v in range(256)))
        SIZE = 64*1024
        cls.large_bdata = b''.join((i2b(randint(0,255)) for v in range(SIZE)))
        
    def testBadFrame(self):
        self.assertRaises(WebSocketProtocolError, Frame, version=809)
        self.assertRaises(WebSocketProtocolError, Frame, opcode=56)
        self.assertRaises(WebSocketProtocolError, Frame)
        self.assertRaises(WebSocketProtocolError, Frame, 'bla',
                          masking_key='foo')
        self.assertRaises(WebSocketProtocolError, Frame, 'bla',
                          masking_key='fooox')
        
    def testControlFrames(self):
        f = Frame.close()
        self.assertEqual(f.opcode, 0x8)
        self.assertTrue(f.payload_length <= 125)
        f = Frame.ping('Hello')
        self.assertEqual(f.opcode, 0x9)
        self.assertEqual(int2bytes(0x89,0x05,0x48,0x65,0x6c,0x6c,0x6f), f.msg)
        self.assertTrue(f.payload_length <= 125)
        r = f.on_received()
        self.assertTrue(r)
        self.assertEqual(r.opcode, 0xA)
        f = Frame.pong()
        self.assertEqual(f.opcode, 0xA)
        self.assertTrue(f.payload_length <= 125)
        
    def testUnmaskedDataFrame(self):
        f = Frame('Hello', final=True)
        self.assertEqual(f.opcode, 0x1)
        self.assertEqual(f.payload_length, 5)
        self.assertFalse(f.masked)
        self.assertEqual(len(f.msg), 7)
        self.assertEqual(int2bytes(0x81,0x05,0x48,0x65,0x6c,0x6c,0x6f), f.msg)
        f1 = Frame('Hel')
        f2 = Frame.continuation('lo', final=True)
        self.assertEqual(int2bytes(0x01,0x03,0x48,0x65,0x6c), f1.msg)
        self.assertEqual(int2bytes(0x80,0x02,0x6c,0x6f), f2.msg)
        
    def testBinaryDataFrame(self):
        f = Frame(self.bdata, opcode=0x2, final=True)
        self.assertEqual(f.opcode, 0x2)
        self.assertEqual(f.payload_length, 256)
        self.assertFalse(f.masked)
        self.assertEqual(len(f.msg), 260)
        self.assertEqual(struct.pack('!BBH',0x82,0x7E,0x0100), f.msg[:4])
        f = Frame(self.large_bdata, opcode=0x2, final=True)
        self.assertEqual(f.opcode, 0x2)
        self.assertEqual(f.payload_length, len(self.large_bdata))
        self.assertFalse(f.masked)
        self.assertEqual(len(f.msg), len(self.large_bdata)+10)
        self.assertEqual(struct.pack('!BBQ',0x82,0x7F,0x0000000000010000),
                         f.msg[:10])
            
    def testMaskData(self):
        masking_key = int2bytes(0x37,0xfa,0x21,0x3d)
        f = Frame('Hello', masking_key=masking_key, final=True)
        self.assertTrue(f.final)
        msg = int2bytes(0x81,0x85,0x37,0xfa,0x21,0x3d,0x7f,0x9f,0x4d,0x51,0x58)
        self.assertTrue(f.masked)
        self.assertEqual(msg, f.msg)
        
    def testParser(self):
        p = FrameParser()
        self.assertEqual(p.execute(b''), None)
        frame = Frame('Hello', final=True)
        self.assertRaises(WebSocketProtocolError, p.execute, frame.msg)
        frame = Frame('Hello', masking_key='ciao', final=True)
        pframe = p.execute(frame.msg)
        self.assertTrue(pframe)
        self.assertEqual(pframe.body, 'Hello')
        self.assertEqual(pframe.payload_length, 5)
        self.assertEqual(pframe.masking_key, b'ciao')
    
    def testParserBinary(self):
        p = FrameParser()
        frame = Frame(self.bdata, opcode=0x2, final=True, masking_key='ciao')
        pframe = p.execute(frame.msg)
        self.assertTrue(pframe)
        self.assertEqual(pframe.payload_length, 256)
        self.assertEqual(pframe.body, self.bdata)
        frame = Frame(self.large_bdata, opcode=0x2, final=True,
                      masking_key='ciao')
        pframe = p.execute(frame.msg)
        self.assertTrue(pframe)
        self.assertEqual(pframe.payload_length, len(self.large_bdata))
        self.assertEqual(pframe.body, self.large_bdata)
        
    def testPartialParsing(self):
        p = FrameParser()
        frame = Frame(self.large_bdata, opcode=0x2, final=True,
                      masking_key='ciao')
        self.assertEqual(p.execute(frame.msg[:1]), None)
        self.assertEqual(p.execute(frame.msg[1:5]), None)
        self.assertEqual(p.execute(frame.msg[5:50]), None)
        pframe = p.execute(frame.msg[50:])
        self.assertTrue(pframe)
        self.assertEqual(pframe.payload_length, len(self.large_bdata))
        self.assertEqual(pframe.body, self.large_bdata)


'''Decorator for RPC functions operating with users and model instances
'''
from pulsar.utils.tools import checkarity

from .exceptions import InvalidParams


def wrap_object_call(fname, namefunc):
    def _(self,*args,**kwargs):
        f = getattr(self, fname)
        return f(*args,**kwargs)
    _.__name__ = namefunc
    return _
    
def rpcerror(func, args, kwargs, discount=0):
    msg = checkarity(func, args, kwargs, discount=discount)
    if msg:
        raise InvalidParams('Invalid Parameters. %s' % msg)
    else:
        raise
    
def FromApi(func, doc=None, format='json', request_handler=None):
    '''A decorator which exposes a function ``func`` as an rpc function.

:parameter func: The function to expose.
:parameter doc: Optional doc string. If not provided the doc string of
    ``func`` will be used.
:parameter format: Optional output format.
:parameter request_handler: function which takes ``request``, ``format`` and
     ``kwargs`` and return a new ``kwargs`` to be passed to
     ``func``. It can be used to add additional parameters based
     on request and format.'''
    def _(self, *args, **kwargs):
        request = args[0]
        if request_handler:
            kwargs = request_handler(request, format, kwargs)
        request.format = kwargs.pop('format', format)
        try:
            return func(*args, **kwargs)
        except TypeError:
            rpcerror(func, args, kwargs)
        
    _.__doc__ = doc or func.__doc__
    _.__name__ = func.__name__
    _.FromApi = True
    return _




def arbiter_task(arbiter):
    import objgraph
    EVERY = 299
    #
    last_show = arbiter.local.get('objgraph') or 0
    if not last_show:
        #objgraph.show_most_common_types()
        arbiter.log.error('Arbiter loops %s' % arbiter.ioloop.num_loops)
        objgraph.show_growth(limit=10)
    last_show += 1
    if last_show > EVERY:
        last_show = 0
    arbiter.local['objgraph'] = last_show


'''A HTTP proxy server in Pulsar::

    python manage.py
'''
import io
import json
import sys
try:
    import pulsar
except ImportError:
    sys.path.append('../../')

from pulsar import HttpException, LocalMixin, HttpClient, maybe_async,\
                    is_async, is_failure, local_property
from pulsar.apps import wsgi
from pulsar.utils.httpurl import Headers

ENVIRON_HEADERS = ('content-type', 'content-length')
USER_AGENT = 'Pulsar-Proxy-Server'


def x_forwarded_for(environ, headers):
    headers.add_header('x-forwarded-for', environ['REMOTE_ADDR'])
    

class ProxyMiddleware(LocalMixin):
    '''WSGI middleware for an asynchronous proxy server. To perform
processing on headers you can pass a list of ``headers_middleware``.
An headers middleware is a callable which accepts two parameters, the wsgi
*environ* dictionary and the *headers* container.'''
    def __init__(self, user_agent=None, headers_middleware=None):
        self.headers = headers = Headers(kind='client')
        self.headers_middleware = headers_middleware or []
        if user_agent:
            headers['user-agent'] = user_agent
            
    @local_property
    def http_client(self):
        return HttpClient(timeout=0, decompress=False, store_cookies=False,
                          stream=True)
        
    def __call__(self, environ, start_response):
        # The WSGI thing
        uri = environ['RAW_URI']
        if not uri or uri.startswith('/'):
            raise HttpException(status=404)
        wsgi_response = wsgi.WsgiResponse(environ=environ,
                                          start_response=start_response)
        headers = self.request_headers(environ)
        method = environ['REQUEST_METHOD']
        stream = environ.get('wsgi.input') or io.BytesIO()
        target_response = self.http_client.request(method, uri,
                                                   data=stream.getvalue(),
                                                   headers=headers)
        wsgi_response.content = self.response_generator(target_response,
                                                        wsgi_response)
        return wsgi_response
    
    def request_headers(self, environ):
        '''Modify request headers via the list of :attr:`headers_middleware`.
The returned headers will be sent to the target uri.'''
        headers = Headers(kind='client')
        for k in environ:
            if k.startswith('HTTP_'):
                head = k[5:].replace('_','-')
                headers[head] = environ[k]
        for head in ENVIRON_HEADERS:
            k = head.replace('-','_').upper()
            v = environ.get(k)
            if v:
                headers[head] = v
        headers.update(self.headers)
        for middleware in self.headers_middleware:
            middleware(environ, headers)
        return headers
        
    def response_generator(self, response, wsgi_response):
        response = maybe_async(response)
        while is_async(response):
            yield b''
            response = maybe_async(response)
        stream_content = None
        if is_failure(response):
            wsgi_response.status_code = 500
        else:
            wsgi_response.status_code = response.status_code
            wsgi_response.headers.update(response.headers)
            stream_content = response.stream()
        wsgi_response.start()
        if stream_content:
            for content in stream_content:
                yield content


def server(description=None, name='proxy-server', **kwargs):
    description = description or 'Pulsar Proxy Server'
    wsgi_proxy = ProxyMiddleware(user_agent=USER_AGENT,
                                 headers_middleware=[x_forwarded_for])
    app = wsgi.WsgiHandler(middleware=[wsgi_proxy])
    return wsgi.WSGIServer(app, name=name, description=description, **kwargs)


if __name__ == '__main__':
    server().start()



'''Configuration utilities. originally from gunicorn_,
adapted and modified for pulsar.

Original Gunicorn Licence

This file is part of gunicorn released under the MIT license.
See the NOTICE for more information.

.. _gunicorn: http://gunicorn.org/
'''
import copy
import inspect
import argparse
import os
import textwrap
import types

from pulsar import __version__, SERVER_NAME, parse_address
from . import system
from .httpurl import to_bytes, iteritems, HttpParser as PyHttpParser,\
                     native_str
from .importer import import_system_file


__all__ = ['Config',
           'SimpleSetting',
           'Setting',
           'defaults',
           'ordered_settings',
           'validate_string',
           'validate_callable',
           'validate_bool',
           'validate_list',
           'validate_pos_int',
           'validate_pos_float',
           'make_settings']

class DefaultSettings:

    def __init__(self):
        # port for serving a socket
        self.PORT = 8060
        # timeout for asynchronous Input/Output
        self.IO_TIMEOUT = None
        # Maximum number of concurrenct clients
        self.BACKLOG = 2048
        # Actors timeout
        self.TIMEOUT = 30
        # mailbox clients blocking
        self.message_timeout = 3

defaults = DefaultSettings()
KNOWN_SETTINGS = {}
KNOWN_SETTINGS_ORDER = []


def def_arity1(server):
    pass

def def_arity2(worker, req):
    pass

def wrap_method(func):
    def _wrapped(instance, *args, **kwargs):
        return func(*args, **kwargs)
    return _wrapped


def ordered_settings():
    for name in KNOWN_SETTINGS_ORDER:
        yield KNOWN_SETTINGS[name]

def make_settings(apps=None, include=None, exclude=None):
    '''Creates a dictionary of available settings for given
applications *apps*.

:parameter apps: Optional list of application names.
:parameter include: Optional list of settings to include.
:parameter exclude: Optional list of settings to exclude.
:rtype: dictionary of :class:`pulsar.Setting` instances.'''
    settings = {}
    include = set(include or ())
    exclude = set(exclude or ())
    apps = set(apps or ())
    for s in ordered_settings():
        setting = s()
        if setting.name not in include:
            if setting.name in exclude:
                continue    # setting name in exclude set
            if setting.app and setting.app not in apps:
                continue    # the setting is for an app not in the apps set
        settings[setting.name] = setting.copy()
    return settings


class Config(object):
    '''Dictionary containing :class:`Setting` parameters for
fine tuning pulsar servers. It provides easy access to :attr:`Setting.value`
attribute by exposing the :attr:`Setting.name` as attribute.

.. attribute:: settings

    Dictionary of all :class:`Settings` instances available. The
    keys are given by the :attr:`Setting.name` attribute.
'''
    exclude_from_config = set(('config',))
    
    def __init__(self, description=None, epilog=None,
                 version=None, app=None, include=None,
                 exclude=None, settings=None):
        if settings is None:
            settings = make_settings(app, include, exclude)
        self.settings = settings
        self.description = description or 'Pulsar server'
        self.epilog = epilog or 'Have fun!'
        self.version = version or __version__
        
    def __iter__(self):
        return iter(self.settings)
    
    def __len__(self):
        return len(self.settings)
    
    def items(self):
        for k, setting in iteritems(self.settings):
            yield k, setting.value
    
    def __getstate__(self):
        return self.__dict__.copy()

    def __setstate__(self, state):
        for k,v in state.items():
            self.__dict__[k] = v

    def __getattr__(self, name):
        return self.get(name)

    def __setattr__(self, name, value):
        if name != "settings" and name in self.settings:
            raise AttributeError("Invalid access!")
        super(Config, self).__setattr__(name, value)

    def get(self, name, default=None):
        if name not in self.settings:
            if name in KNOWN_SETTINGS:
                return default
            raise AttributeError("No configuration setting for: %s" % name)
        return self.settings[name].get()

    def set(self, name, value):
        if name not in self.settings:
            raise AttributeError("No configuration setting for: %s" % name)
        self.settings[name].set(value)

    def parser(self):
        '''Create the argparser_ for this configuration by adding all
settings via the :meth:`Setting.add_argument`.

:rtype: an instance of :class:`argparse.ArgumentParser`.

.. _argparser: http://docs.python.org/dev/library/argparse.html
'''
        kwargs = {
            "description": self.description,
            "epilog": self.epilog
        }
        parser = argparse.ArgumentParser(**kwargs)
        parser.add_argument('--version',
                            action='version',
                            version=self.version)
        setts = self.settings
        sorter = lambda x: (setts[x].section, setts[x].order)
        for k in sorted(setts, key=sorter):
            setts[k].add_argument(parser)
        return parser

    def import_from_module(self, mod=None):
        if mod:
            self.set('config', mod)
        try:
            mod = import_system_file(self.config)
        except Exception as e:
            raise RuntimeError('Failed to read config file "%s". %s' %\
                               (self.config, e))
        unknowns = []
        if mod:
            for k in dir(mod):
                # Skip private functions and attributes
                kl = k.lower()
                if k.startswith('_') or kl in self.exclude_from_config:
                    continue
                val = getattr(mod, k)
                # add unknown names to list
                if kl not in self.settings:
                    unknowns.append((k, val))
                else:
                    self.set(kl, val)
        return unknowns
        
    def on_start(self):
        '''Invoked by a :class:`pulsar.Application` just before starting.'''
        for sett in self.settings.values():
            sett.on_start()

    @property
    def workers(self):
        return self.settings['workers'].get()

    @property
    def address(self):
        bind = self.settings['bind']
        if bind:
            return parse_address(to_bytes(bind.get()))

    @property
    def uid(self):
        user = self.settings.get('user')
        if user:
            return system.get_uid(user.get())

    @property
    def gid(self):
        group = self.settings.get('group')
        if group:
            return system.get_gid(group.get())

    @property
    def proc_name(self):
        pn = self.settings.get('process_name')
        if pn:
            pn = pn.get()
        if pn is not None:
            return pn
        else:
            pn = self.settings.get('default_process_name')
            if pn:
                return pn.get()
        
    def copy(self):
        cls = self.__class__
        me = cls.__new__(cls)
        for name, value in iteritems(self.__dict__):
            if name == 'settings':
                value = copy.deepcopy(value)
            me.__dict__[name] = value
        return me        
    
    def __copy__(self):
        return self.copy()
    
    def __deepcopy__(self, memo):
        return self.__copy__()
    
    def new_config(self):
        cfg = self.__class__()
        for key, sett in iteritems(self.settings):
            if key in cfg.settings and sett.inherit:
                cfg.set(key, sett.value)
        return cfg
            

class SettingMeta(type):
    '''A metaclass which collects all setting classes and put them
in the global ``KNOWN_SETTINGS`` list.'''
    def __new__(cls, name, bases, attrs):
        super_new = super(SettingMeta, cls).__new__
        parents = [b for b in bases if isinstance(b, SettingMeta)]
        val = attrs.get("validator")
        attrs["validator"] = wrap_method(val) if val else None
        if attrs.pop('virtual', False):
            return super_new(cls, name, bases, attrs)
        attrs["order"] = len(KNOWN_SETTINGS) + 1
        new_class = super_new(cls, name, bases, attrs)
        # build one instance to increase count
        new_class()
        new_class.fmt_desc(attrs['desc'] or '')
        if not new_class.name:
            new_class.name = new_class.__name__.lower()
        if new_class.name in KNOWN_SETTINGS_ORDER:
            old_class = KNOWN_SETTINGS.pop(new_class.name)
            new_class.order = old_class.order
        else:
            KNOWN_SETTINGS_ORDER.append(new_class.name)
        KNOWN_SETTINGS[new_class.name] = new_class
        return new_class

    def fmt_desc(cls, desc):
        desc = textwrap.dedent(desc).strip()
        setattr(cls, "desc", desc)
        lines = desc.splitlines()
        setattr(cls, "short", '' if not lines else lines[0])


class SettingBase(object):
    inherit = True
    name = None
    '''The unique name used to access this setting.'''
    validator = None
    '''Validator for this setting. It provided it must be a fucntion
accepting one positional argument, the value to validate.'''
    value = None
    '''The actual value for this setting.'''
    def __str__(self):
        return '{0} ({1})'.format(self.name, self.value)
    __repr__ = __str__

    def set(self, val):
        '''Set *val* as the value in this :class:`Setting`.'''
        if hasattr(self.validator,'__call__'):
            val = self.validator(val)
        self.value = val

    def get(self):
        return self.value

    def on_start(self):
        '''Called when pulsar server starts. It can be used to perform
custom initialization for this :class:`Setting`.'''
        pass


class SimpleSetting(SettingBase):

    def __init__(self, name, value):
        self.name = name
        self.value = value


# This works for Python 2 and Python 3
class Setting(SettingMeta('BaseSettings', (SettingBase,), {'virtual': True})):
    '''A configuration parameter for pulsar. Parameters can be specified
on the command line or on a config file.'''
    creation_count = 0
    virtual = True
    '''If set to ``True`` the settings won't be loaded and it can be only used
as base class for other settings.'''
    nargs = None
    '''For positional arguments. Same usage as argparse.'''
    app = None
    '''Setting for a specific :class:`Application`.'''
    section = None
    '''Setting section, used for creating documentation.'''
    flags = None
    '''List of options strings, e.g. ``[-f, --foo]``.'''
    choices = None
    '''Restrict the argument to the choices provided.'''
    type = None
    meta = None
    action = None
    default = None
    '''Default value'''
    short = None
    desc = None

    def __init__(self, name=None, flags=None, action=None, type=None,
                 default=None, nargs=None, desc=None, validator=None):
        self.default = default if default is not None else self.default
        if self.default is not None:
            self.set(self.default)
        self.name = name or self.name
        self.flags = flags or self.flags
        self.action = action or self.action
        self.nargs = nargs or self.nargs
        self.type = type or self.type
        self.desc = desc or self.desc
        self.short = self.short or self.desc
        self.desc = self.desc or self.short
        #if validator:
        #    self.validator = wrap_method(validator)
        if self.app and not self.section:
            self.section = self.app
        if not self.section:
            self.section = 'unknown'
        self.__class__.creation_count += 1
        if not hasattr(self, 'order'):
            self.order = 1000 + self.__class__.creation_count

    def __getstate__(self):
        return self.__dict__.copy()

    def add_argument(self, parser):
        '''Add itself to the argparser.'''
        kwargs = {'nargs':self.nargs}
        if self.type and self.type != 'string':
            kwargs["type"] = self.type
        if self.flags:
            args = tuple(self.flags)
            kwargs.update({"dest": self.name,
                           "action": self.action or "store",
                           "default": None,
                           "help": "%s [%s]" % (self.short, self.default)})
            if kwargs["action"] != "store":
                kwargs.pop("type",None)
                kwargs.pop("nargs",None)
        elif self.nargs and self.name:
            args = (self.name,)
            kwargs.update({'metavar': self.meta or None,
                           'help': self.short})
        else:
            # Not added to argparser
            return
        parser.add_argument(*args, **kwargs)

    def copy(self):
        return copy.copy(self)


def validate_bool(val):
    if isinstance(val, bool):
        return val
    val = native_str(val)
    if not isinstance(val, str):
        raise TypeError("Invalid type for casting: %s" % val)
    if val.lower().strip() == "true":
        return True
    elif val.lower().strip() == "false":
        return False
    else:
        raise ValueError("Invalid boolean: %s" % val)

def validate_pos_int(val):
    if not isinstance(val, int):
        val = int(val, 0)
    else:
        # Booleans are ints!
        val = int(val)
    if val < 0:
        raise ValueError("Value must be positive: %s" % val)
    return val

def validate_pos_float(val):
    val = float(val)
    if val < 0:
        raise ValueError("Value must be positive: %s" % val)
    return val

def validate_string(val):
    va = native_str(val)
    if va is None:
        return None
    if not isinstance(va, str):
        raise TypeError("Not a string: %s" % val)
    return va.strip()

def validate_list(val):
    if val and not isinstance(val, (list, tuple)):
        raise TypeError("Not a list: %s" % val)
    return list(val)

def validate_dict(val):
    if val and not isinstance(val, dict):
        raise TypeError("Not a dictionary: %s" % val)
    return val

def validate_callable(arity):
    def _validate_callable(val):
        if not hasattr(val,'__call__'):
            raise TypeError("Value is not callable: %s" % val)
        if not inspect.isfunction(val):
            cval = val.__call__
            discount = 1
        else:
            discount = 0
            cval = val
        if arity != len(inspect.getargspec(cval)[0]) - discount:
            raise TypeError("Value must have an arity of: %s" % arity)
        return val
    return _validate_callable


class ConfigFile(Setting):
    name = "config"
    section = "Config File"
    flags = ["-c", "--config"]
    meta = "FILE"
    validator = validate_string
    default = 'config.py'
    desc = """\
        The path to a Pulsar config file.

        Only has an effect when specified on the command line or as part of an
        application specific configuration.
        """


class Workers(Setting):
    inherit = False     # not ineritable by the arbiter
    name = "workers"
    section = "Worker Processes"
    flags = ["-w", "--workers"]
    validator = validate_pos_int
    type = int
    default = 1
    desc = """\
        The number of workers for handling requests.

If you are using a multi-process concurrency, a number in the
the 2-4 x $(NUM_CORES) range should be good. If you are using threads this
number can be higher."""


class Concurrency(Setting):
    name = "concurrency"
    section = "Worker Processes"
    flags = ["--concurrency"]
    default = "process"
    desc = """\
        The type of concurrency to use: ``process`` or ``thread``.
        """


class MaxRequests(Setting):
    inherit = False     # not ineritable by the arbiter
    name = "max_requests"
    section = "Worker Processes"
    flags = ["--max-requests"]
    validator = validate_pos_int
    type = int
    default = 0
    desc = """\
        The maximum number of requests a worker will process before restarting.

        Any value greater than zero will limit the number of requests a work
        will process before automatically restarting. This is a simple method
        to help limit the damage of memory leaks.

        If this is set to zero (the default) then the automatic worker
        restarts are disabled.
        """

class Backlog(Setting):
    inherit = False     # not ineritable by the arbiter
    name = "backlog"
    section = "Worker Processes"
    flags = ["--backlog"]
    validator = validate_pos_int
    type = int
    default = 2048
    desc = """\
        The maximum number of concurrent requests.
        This refers to the number of clients that can be waiting to be served.
        Exceeding this number results in the client getting an error when
        attempting to connect. It should only affect servers under significant
        load.

        Must be a positive integer. Generally set in the 64-2048 range.
        """


class Timeout(Setting):
    inherit = False     # not ineritable by the arbiter
    name = "timeout"
    section = "Worker Processes"
    flags = ["-t", "--timeout"]
    validator = validate_pos_int
    type = int
    default = 30
    desc = """\
        Workers silent for more than this many seconds are killed and restarted.

        Generally set to thirty seconds. Only set this noticeably higher if
        you're sure of the repercussions for sync workers. For the non sync
        workers it just means that the worker process is still communicating and
        is not tied to the length of time required to handle a single request.
        """


class HttpProxyServer(Setting):
    name = "http_proxy"
    section = "Http"
    flags = ["--http-proxy"]
    default = ''
    desc = """\
        The HTTP proxy server to use with HttpClient.
        """
    def on_start(self):
        if self.value:
            os.environ['http_proxy'] = self.value
            os.environ['https_proxy'] = self.value


class HttpParser(Setting):
    name = "http_py_parser"
    section = "Http"
    flags = ["--http-py-parser"]
    action = "store_true"
    default = False
    desc = """\
    Set the python parser as default HTTP parser.
        """

    def on_start(self):
        if self.value:
            from pulsar.lib import setDefaultHttpParser 
            setDefaultHttpParser(PyHttpParser)


class Debug(Setting):
    name = "debug"
    section = "Debugging"
    flags = ["--debug"]
    validator = validate_bool
    action = "store_true"
    default = False
    desc = """\
        Turn on debugging in the server.

        This limits the number of worker processes to 1 and changes some error
        handling that's sent to clients.
        """


class Daemon(Setting):
    name = "daemon"
    section = "Server Mechanics"
    flags = ["-D", "--daemon"]
    validator = validate_bool
    action = "store_true"
    default = False
    desc = """\
        Daemonize the Pulsar process (posix only).

        Detaches the server from the controlling terminal and enters the
        background.
        """


class Pidfile(Setting):
    name = "pidfile"
    section = "Server Mechanics"
    flags = ["-p", "--pid"]
    meta = "FILE"
    validator = validate_string
    default = None
    desc = """\
        A filename to use for the PID file.

        If not set, no PID file will be written.
        """


class Password(Setting):
    name = "password"
    section = "Server Mechanics"
    flags = ["--password"]
    validator = validate_string
    default = None
    desc = """Set a password for the server"""


class User(Setting):
    name = "user"
    section = "Server Mechanics"
    flags = ["-u", "--user"]
    meta = "USER"
    validator = validate_string
    default = None
    desc = """\
        Switch worker processes to run as this user.

        A valid user id (as an integer) or the name of a user that can be
        retrieved with a call to pwd.getpwnam(value) or None to not change
        the worker process user.
        """


class Group(Setting):
    name = "group"
    section = "Server Mechanics"
    flags = ["-g", "--group"]
    meta = "GROUP"
    validator = validate_string
    default = None
    desc = """\
        Switch worker process to run as this group.

        A valid group id (as an integer) or the name of a user that can be
        retrieved with a call to pwd.getgrnam(value) or None to not change
        the worker processes group.
        """


class Loglevel(Setting):
    name = "loglevel"
    section = "Logging"
    flags = ["--log-level"]
    meta = "LEVEL"
    validator = validate_string
    default = "info"
    desc = """The granularity of log outputs.
            
            Valid level names are:
            
             * debug
             * info
             * warning
             * error
             * critical
             """

class LogHandlers(Setting):
    name = "loghandlers"
    section = "Logging"
    flags = ["--log-handlers"]
    default = ['console']
    validator = validate_list
    desc = """log handlers for pulsar server"""
    
    
class LogEvery(Setting):
    name = "logevery"
    section = "Logging"
    flags = ["--log-every"]
    validator = validate_pos_int
    default = 0
    desc = """Log information every n seconds"""


class LogConfig(Setting):
    name = "logconfig"
    section = "Logging"
    default = {}
    validator = validate_dict
    desc = '''
    The logging configuration dictionary.

    This settings can only be specified on a config file
    '''


class Procname(Setting):
    name = "process_name"
    section = "Process Naming"
    flags = ["-n", "--name"]
    meta = "STRING"
    validator = validate_string
    default = None
    desc = """\
        A base to use with setproctitle for process naming.

        This affects things like ``ps`` and ``top``. If you're going to be
        running more than one instance of Pulsar you'll probably want to set a
        name to tell them apart. This requires that you install the setproctitle
        module.

        It defaults to 'pulsar'.
        """


class DefaultProcName(Setting):
    name = "default_process_name"
    section = "Process Naming"
    validator = validate_string
    default = SERVER_NAME
    desc = """\
        Internal setting that is adjusted for each type of application.
        """


class WhenReady(Setting):
    name = "when_ready"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """\
        Called just after the server is started.

        The callable needs to accept a single instance variable for the Arbiter.
        """


class Prefork(Setting):
    name = "pre_fork"
    section = "Server Hooks"
    validator = validate_callable(1)
    default = staticmethod(def_arity1)
    type = "callable"
    desc = """\
        Called just before a worker is forked.

        The callable needs to accept two instance variables for the Arbiter and
        new Worker.
        """


class Postfork(Setting):
    name = "post_fork"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """\
        Called just after a worker has been forked.

        The callable needs to accept two instance variables for the Arbiter and
        new Worker.
        """


class PreExec(Setting):
    name = "pre_exec"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """\
        Called just before a new master process is forked.

        The callable needs to accept a single instance variable for the Arbiter.
        """


class PreRequest(Setting):
    name = "pre_request"
    section = "Server Hooks"
    validator = validate_callable(2)
    type = "callable"
    default = staticmethod(def_arity2)
    desc = """\
        Called just before a worker processes the request.

        The callable needs to accept two instance variables for the Worker and
        the Request.
        """


class PostRequest(Setting):
    name = "post_request"
    section = "Server Hooks"
    validator = validate_callable(2)
    type = "callable"
    default = staticmethod(def_arity2)
    desc = """\
        Called after a worker processes the request.

        The callable needs to accept two instance variables for the Worker and
        the Request.
        """


class WorkerExit(Setting):
    name = "worker_exit"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """Called just after a worker has been exited.

        The callable needs to accept one variable for the
        the just-exited Worker.
        """


class WorkerTask(Setting):
    name = "worker_task"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """Called at every event loop by the worker.

        The callable needs to accept one variable for the Worker.
        """
        
        
class ArbiterTask(Setting):
    name = "arbiter_task"
    section = "Server Hooks"
    validator = validate_callable(1)
    type = "callable"
    default = staticmethod(def_arity1)
    desc = """Called at every event loop by the arbiter.

        The callable needs to accept one variable for the Arbiter.
        """


import sys
import inspect
import logging

from pulsar import to_bytes, is_failure, log_failure, is_async,\
                    as_failure, maybe_async, HttpException
from pulsar.utils.tools import checkarity
from pulsar.utils.structures import AttributeDictionary
from pulsar.apps.wsgi import WsgiResponse, WsgiResponseGenerator, Request

from .decorators import rpcerror, wrap_object_call
from .exceptions import *


__all__ = ['RpcMiddleware']

LOGGER = logging.getLogger('pulsar.rpc')

class RPC:
    
    def __init__(self, handler, method, func, args, kwargs,
                 id=None, version=None):
        self.handler = handler
        self.method = method
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.id = id
        self.version = version
        
    def process(self, request):
        func = self.func
        if not func:
            raise NoSuchFunction('Function "%s" not available.' % self.method)
        try:
            return func(self.handler, request, *self.args, **self.kwargs)
        except TypeError as e:
            if not getattr(func, 'FromApi', False):
                rpcerror(func, self.args, self.kwargs, discount=2)
            else:
                raise
    

class ResponseGenerator(WsgiResponseGenerator):
    '''Asynchronous response generator invoked by the djpcms WSGI middleware'''
    def __init__(self, request, start_response):
        self.request = request
        super(ResponseGenerator, self).__init__(request.environ, start_response)

    def __iter__(self):
        request = self.request
        rpc = request['rpc']
        status_code = 200
        try:
            result = rpc.process(request)
        except Exception as e:
            result = as_failure(e)
        handler = rpc.handler
        result = maybe_async(result)
        while is_async(result):
            yield b''
            result = maybe_async(result)
        try:
            if is_failure(result):
                e = result.trace[1]
                status_code = getattr(e, 'status', 400)
                log_failure(result)
                result = handler.dumps(rpc.id, rpc.version, error=e)
            else:
                result = handler.dumps(rpc.id, rpc.version, result=result)
        except Exception as e:
            LOGGER.error('Could not serialize', exc_info=True)
            status_code = 500
            result = handler.dumps(rpc.id, rpc.version, error=e)
        content = to_bytes(result)
        response = WsgiResponse(status_code, content,
                                content_type=handler.content_type)
        for c in self.start(response):
            yield c


class MetaRpcHandler(type):
    '''A metaclass for rpc handlers.
Add a limited ammount of magic to RPC handlers.'''
    def __new__(cls, name, bases, attrs):
        make = super(MetaRpcHandler, cls).__new__
        if attrs.pop('virtual',None):
            return make(cls,name,bases,attrs)
        funcprefix = attrs.get('serve_as',None)
        if not funcprefix:
            for base in bases[::-1]:
                if isinstance(base, MetaRpcHandler):
                    funcprefix = base.serve_as
                    if funcprefix:
                        break
        rpc = {}
        if funcprefix:
            fprefix = '%s_' % funcprefix
            for key, method in list(attrs.items()):
                if hasattr(method,'__call__') and key.startswith(fprefix):
                    namefunc = key[len(fprefix):]
                    func = attrs.pop(key)
                    if not inspect.isfunction(func):
                        key = '_{0}'.format(key)
                        attrs[key] = func
                        func = wrap_object_call(key, namefunc)
                    rpc[namefunc] = func
            for base in bases[::-1]:
                if hasattr(base, 'rpcfunctions'):
                    rpcbase = base.rpcfunctions
                    for key,method in rpcbase.items():
                        if key not in rpc:
                            rpc[key] = method

        attrs['rpcfunctions'] = rpc
        return make(cls, name, bases, attrs)


class RpcHandler(MetaRpcHandler('_RpcHandler', (object,), {'virtual': True})):
    '''The base class for rpc handlers.

.. attribute:: content_type

    Default content type. Default: ``text/plain``.

.. attribute:: charset

    The default charset for this handler. Default: ``utf-8``.
'''
    serve_as     = 'rpc'
    '''Prefix for class methods providing remote services. Default: ``rpc``.'''
    separator    = '.'
    '''Separator between :attr:`subHandlers`.'''
    content_type = 'text/plain'
    methods = ('get','post','put','head','delete','trace','connect')
    charset = 'utf-8'
    _info_exceptions = (Fault,)

    def __init__(self, subhandlers=None, title=None, documentation=None):
        self._parent = None
        self.subHandlers = {}
        self.title = title or self.__class__.__name__
        self.documentation = documentation or ''
        if subhandlers:
            for prefix,handler in subhandlers.items():
                if inspect.isclass(handler):
                    handler = handler()
                self.putSubHandler(prefix, handler)

    @property
    def parent(self):
        '''The parent :class:`RpcHandler` or ``None`` if this
is the root handler.'''
        return self._parent

    @property
    def root(self):
        '''The root :class:`RpcHandler` or ``self`` if this
is the root handler.'''
        return self._parent.root if self._parent is not None else self

    def isroot(self):
        '''``True`` if this is the root handler.'''
        return self._parent == None

    def get_method_and_args(self, data):
        '''Obtain function information form ``wsgi.input``. Needs to be
implemented by subclasses. It should return a five elements tuple containing::

    method, args, kwargs, id, version

where ``method`` is the function name, ``args`` are positional parameters
for ``method``, ``kwargs`` are keyworded parameters for ``method``,
``id`` is an identifier for the client,
``version`` is the version of the RPC protocol.
    '''
        raise NotImplementedError()


    def __getstate__(self):
        d = self.__dict__.copy()
        if not self.isroot():
            # Avoid duplicating handlers
            d['_parent'] = True
        return d

    def __setstate__(self, state):
        self.__dict__ = state
        for handler in self.subHandlers.values():
            handler._parent = self

    def putSubHandler(self, prefix, handler):
        '''Add a sub :class:`RpcHandler` with prefix ``prefix``.

:keyword prefix: a string defining the prefix of the subhandler
:keyword handler: the sub-handler.
        '''
        self.subHandlers[prefix] = handler
        handler._parent = self
        return self

    def getSubHandler(self, prefix):
        '''Get a sub :class:`RpcHandler` at ``prefix``.'''
        return self.subHandlers.get(prefix)

    def wrap_function_decorator(self, request, *args, **kwargs):
        return request.func(rpc.handler, request, *args,**kwargs)

    def request(self, environ, method, args, kwargs, id, version):
        bits = method.split(self.separator, 1)
        handler = self
        method_name = bits[-1]
        for bit in bits[:-1]:
            subhandler = handler.getSubHandler(bit)
            if subhandler is None:
                method_name = method
                break
            else:
                handler = subhandler
        try:
            func = handler.rpcfunctions[method_name]
        except:
            func = None
        environ['rpc'] = RPC(handler, method, func, args, kwargs, id, version)

    def invokeServiceEndpoint(self, meth, args):
        return meth(*args)

    def listFunctions(self, prefix = ''):
        for name,func in self.rpcfunctions.items():
            doc = {'doc':func.__doc__ or 'No docs','section':prefix}
            yield '{0}{1}'.format(prefix,name),doc
        for name,handler in self.subHandlers.items():
            pfx = '{0}{1}{2}'.format(prefix,name,self.separator)
            for f,doc in handler.listFunctions(pfx):
                yield f,doc

    def _docs(self):
        for name, data in self.listFunctions():
            link = '.. _functions-{0}:'.format(name)
            title = name
            under = (2+len(title))*'-'
            yield '\n'.join((link,'',title,under,'',data['doc'],'\n'))

    def docs(self):
        return '\n'.join(self._docs())


class RpcMiddleware(object):
    '''A WSGI_ middleware for serving an :class:`RpcHandler`.

.. attribute:: handler

    The :class:`RpcHandler` to serve.

.. attribute:: path

    The path where the RPC is located

    Default ``None``
'''
    request_class = Request

    def __init__(self, handler, path=None):
        self.handler = handler
        self.path = path or '/'

    def __str__(self):
        return self.path

    def __repr__(self):
        return '{0}({1})'.format(self.__class__.__name__,self)

    @property
    def route(self):
        return self.path

    def __call__(self, environ, start_response):
        '''The WSGI handler which consume the remote procedure call'''
        path = environ['PATH_INFO'] or '/'
        if path == self.path:
            method = environ['REQUEST_METHOD'].lower()
            if method not in self.handler.methods:
                raise HttpException(status=405, msg='Method "%s" not allowed' %\
                                    method)
            data = environ['wsgi.input'].read()
            hnd = self.handler
            method, args, kwargs, id, version = hnd.get_method_and_args(data)
            hnd.request(environ, method, args, kwargs, id, version)
            request = self.request_class(environ)
            return ResponseGenerator(request, start_response)

